@@ -62,6 +62,10 @@
  * 06.11.96  - Added v2 code submitted by Joerg Dorchain, but written by
  *             Andreas Schwab.
  *
+ * 1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+ * - added Native Language Support
+ *
+ *
  * I've had no time to add comments - hopefully the function names
  * are comments enough. As with all file system checkers, this assumes
  * the file system is quiescent - don't use it on a mounted device
@@ -92,9 +96,8 @@
 #include <mntent.h>
 #include <sys/stat.h>
 
-#include <linux/fs.h>
-#include <linux/minix_fs.h>
-#include "../version.h"
+#include "minix.h"
+#include "nls.h"
 
 #ifdef MINIX2_SUPER_MAGIC2
 #define HAVE_MINIX2 1
@@ -120,7 +123,6 @@
 #define BITS_PER_BLOCK (BLOCK_SIZE<<3)
 
 static char * program_name = "fsck.minix";
-static char * program_version = "1.2 - 11/11/96";
 static char * device_name = NULL;
 static int IN;
 static int repair=0, automatic=0, verbose=0, list=0, show=0, warn_mode=0, 
@@ -166,8 +168,10 @@ static char *zone_map;
 static unsigned char * inode_count = NULL;
 static unsigned char * zone_count = NULL;
 
-void recursive_check(unsigned int ino);
-void recursive_check2(unsigned int ino);
+static void recursive_check(unsigned int ino);
+#ifdef HAVE_MINIX2
+static void recursive_check2(unsigned int ino);
+#endif
 
 #include "bitops.h"
 
@@ -180,34 +184,43 @@ void recursive_check2(unsigned int ino);
 #define mark_zone(x) (setbit(zone_map,(x)-FIRSTZONE+1),changed=1)
 #define unmark_zone(x) (clrbit(zone_map,(x)-FIRSTZONE+1),changed=1)
 
-void fatal_error(const char *, int) __attribute__ ((noreturn));
-void fatal_error(const char * fmt_string, int status)
-{
-	fprintf(stderr,fmt_string,program_name,device_name);
+static void
+leave(int status) {
 	if (termios_set)
 		tcsetattr(0, TCSANOW, &termios);
 	exit(status);
 }
 
-#define usage() fatal_error("Usage: %s [-larvsmf] /dev/name\n",16)
-#define die(str) fatal_error("%s: " str "\n",8)
+static void
+usage(void) {
+	fprintf(stderr,
+		_("Usage: %s [-larvsmf] /dev/name\n"),
+		program_name);
+	leave(16);
+}
+
+static void
+die(const char *str) {
+	fprintf(stderr, "%s: %s\n", program_name, str);
+	leave(8);
+}
 
 /*
  * This simply goes through the file-name data and prints out the
  * current file.
  */
-void print_current_name(void)
-{
-	int i=0;
+static void
+print_current_name(void) {
+	int i = 0;
 
-	while (i<name_depth)
-		printf("/%.*s",namelen,name_list[i++]);
+	while (i < name_depth)
+		printf("/%.*s", namelen, name_list[i++]);
 	if (i == 0)
 		printf ("/");
 }
 
-int ask(const char * string,int def)
-{
+static int
+ask(const char * string, int def) {
 	int c;
 
 	if (!repair) {
@@ -253,8 +266,8 @@ int ask(const char * string,int def)
  * mounted partition.  Code adapted from e2fsck, Copyright (C) 1993,
  * 1994 Theodore Ts'o.  Also licensed under GPL.
  */
-static void check_mount(void)
-{
+static void
+check_mount(void) {
 	FILE * f;
 	struct mntent * mnt;
 	int cont;
@@ -280,13 +293,13 @@ static void check_mount(void)
 	else
 		close(fd);
 	
-	printf ("%s is mounted.	 ", device_name);
+	printf (_("%s is mounted.	 "), device_name);
 	if (isatty(0) && isatty(1))
-		cont = ask("Do you really want to continue", 0);
+		cont = ask(_("Do you really want to continue"), 0);
 	else
 		cont = 0;
 	if (!cont) {
-		printf ("check aborted.\n");
+		printf (_("check aborted.\n"));
 		exit (0);
 	}
 	return;
@@ -298,62 +311,63 @@ static void check_mount(void)
  * if an error was corrected, and returns the zone (0 for no zone
  * or a bad zone-number).
  */
-int check_zone_nr(unsigned short * nr, int * corrected)
-{
+static int
+check_zone_nr(unsigned short * nr, int * corrected) {
 	if (!*nr)
 		return 0;
 	if (*nr < FIRSTZONE)
-		printf("Zone nr < FIRSTZONE in file `");
+		printf(_("Zone nr < FIRSTZONE in file `"));
 	else if (*nr >= ZONES)
-		printf("Zone nr >= ZONES in file `");
+		printf(_("Zone nr >= ZONES in file `"));
 	else
 		return *nr;
 	print_current_name();
 	printf("'.");
-	if (ask("Remove block",1)) {
+	if (ask(_("Remove block"),1)) {
 		*nr = 0;
 		*corrected = 1;
 	}
 	return 0;
 }
 
-
-int check_zone_nr2 (unsigned int *nr, int *corrected)
-{
+#ifdef HAVE_MINIX2
+static int
+check_zone_nr2 (unsigned int *nr, int *corrected) {
 	if (!*nr)
 		return 0;
 	if (*nr < FIRSTZONE)
-		printf ("Zone nr < FIRSTZONE in file `");
+		printf (_("Zone nr < FIRSTZONE in file `"));
 	else if (*nr >= ZONES)
-		printf ("Zone nr >= ZONES in file `");
+		printf (_("Zone nr >= ZONES in file `"));
 	else
 		return *nr;
 	print_current_name ();
 	printf ("'.");
-	if (ask ("Remove block", 1)) {
+	if (ask (_("Remove block"), 1)) {
 		*nr = 0;
 		*corrected = 1;
 	}
 	return 0;
 }
+#endif
 
 /*
  * read-block reads block nr into the buffer at addr.
  */
-void read_block(unsigned int nr, char * addr)
-{
+static void
+read_block(unsigned int nr, char * addr) {
 	if (!nr) {
 		memset(addr,0,BLOCK_SIZE);
 		return;
 	}
 	if (BLOCK_SIZE*nr != lseek(IN, BLOCK_SIZE*nr, SEEK_SET)) {
-		printf("Read error: unable to seek to block in file '");
+		printf(_("Read error: unable to seek to block in file '"));
 		print_current_name();
 		printf("'\n");
 		memset(addr,0,BLOCK_SIZE);
 		errors_uncorrected = 1;
 	} else if (BLOCK_SIZE != read(IN, addr, BLOCK_SIZE)) {
-		printf("Read error: bad block in file '");
+		printf(_("Read error: bad block in file '"));
 		print_current_name();
 		printf("'\n");
 		memset(addr,0,BLOCK_SIZE);
@@ -364,20 +378,20 @@ void read_block(unsigned int nr, char * addr)
 /*
  * write_block writes block nr to disk.
  */
-void write_block(unsigned int nr, char * addr)
-{
+static void
+write_block(unsigned int nr, char * addr) {
 	if (!nr)
 		return;
 	if (nr < FIRSTZONE || nr >= ZONES) {
-		printf("Internal error: trying to write bad block\n"
-		"Write request ignored\n");
+		printf(_("Internal error: trying to write bad block\n"
+		"Write request ignored\n"));
 		errors_uncorrected = 1;
 		return;
 	}
 	if (BLOCK_SIZE*nr != lseek(IN, BLOCK_SIZE*nr, SEEK_SET))
-		die("seek failed in write_block");
+		die(_("seek failed in write_block"));
 	if (BLOCK_SIZE != write(IN, addr, BLOCK_SIZE)) {
-		printf("Write error: bad block in file '");
+		printf(_("Write error: bad block in file '"));
 		print_current_name();
 		printf("'\n");
 		errors_uncorrected = 1;
@@ -389,8 +403,8 @@ void write_block(unsigned int nr, char * addr)
  * It sets 'changed' if the inode has needed changing, and re-writes
  * any indirect blocks with errors.
  */
-int map_block(struct minix_inode * inode, unsigned int blknr)
-{
+static int
+map_block(struct minix_inode * inode, unsigned int blknr) {
 	unsigned short ind[BLOCK_SIZE>>1];
 	unsigned short dind[BLOCK_SIZE>>1];
 	int blk_chg, block, result;
@@ -424,8 +438,8 @@ int map_block(struct minix_inode * inode, unsigned int blknr)
 }
 
 #ifdef HAVE_MINIX2
-int map_block2 (struct minix2_inode *inode, unsigned int blknr)
-{
+static int
+map_block2 (struct minix2_inode *inode, unsigned int blknr) {
   	unsigned int ind[BLOCK_SIZE >> 2];
 	unsigned int dind[BLOCK_SIZE >> 2];
 	unsigned int tind[BLOCK_SIZE >> 2];
@@ -482,8 +496,8 @@ int map_block2 (struct minix2_inode *inode, unsigned int blknr)
 }
 #endif
 
-void write_super_block(void)
-{
+static void
+write_super_block(void) {
 	/*
 	 * Set the state of the filesystem based on whether or not there
 	 * are uncorrected errors.  The filesystem valid flag is
@@ -496,27 +510,27 @@ void write_super_block(void)
 		Super.s_state &= ~MINIX_ERROR_FS;
 	
 	if (BLOCK_SIZE != lseek(IN, BLOCK_SIZE, SEEK_SET))
-		die("seek failed in write_super_block");
+		die(_("seek failed in write_super_block"));
 	if (BLOCK_SIZE != write(IN, super_block_buffer, BLOCK_SIZE))
-		die("unable to write super-block");
+		die(_("unable to write super-block"));
 
 	return;
 }
 
-void write_tables(void)
-{
+static void
+write_tables(void) {
 	write_super_block();
 
 	if (IMAPS*BLOCK_SIZE != write(IN,inode_map,IMAPS*BLOCK_SIZE))
-		die("Unable to write inode map");
+		die(_("Unable to write inode map"));
 	if (ZMAPS*BLOCK_SIZE != write(IN,zone_map,ZMAPS*BLOCK_SIZE))
-		die("Unable to write zone map");
+		die(_("Unable to write zone map"));
 	if (INODE_BUFFER_SIZE != write(IN,inode_buffer,INODE_BUFFER_SIZE))
-		die("Unable to write inodes");
+		die(_("Unable to write inodes"));
 }
 
-void get_dirsize (void)
-{
+static void
+get_dirsize (void) {
 	int block;
 	char blk[BLOCK_SIZE];
 	int size;
@@ -538,12 +552,12 @@ void get_dirsize (void)
 	/* use defaults */
 }
 
-void read_superblock(void)
-{
+static void
+read_superblock(void) {
 	if (BLOCK_SIZE != lseek(IN, BLOCK_SIZE, SEEK_SET))
-		die("seek failed");
+		die(_("seek failed"));
 	if (BLOCK_SIZE != read(IN, super_block_buffer, BLOCK_SIZE))
-		die("unable to read super block");
+		die(_("unable to read super block"));
 	if (MAGIC == MINIX_SUPER_MAGIC) {
 		namelen = 14;
 		dirsize = 16;
@@ -563,20 +577,20 @@ void read_superblock(void)
 		version2 = 1;
 #endif
 	} else
-		die("bad magic number in super-block");
+		die(_("bad magic number in super-block"));
 	if (ZONESIZE != 0 || BLOCK_SIZE != 1024)
-		die("Only 1k blocks/zones supported");
+		die(_("Only 1k blocks/zones supported"));
 	if (IMAPS * BLOCK_SIZE * 8 < INODES + 1)
-		die("bad s_imap_blocks field in super-block");
+		die(_("bad s_imap_blocks field in super-block"));
 	if (ZMAPS * BLOCK_SIZE * 8 < ZONES - FIRSTZONE + 1)
-		die("bad s_zmap_blocks field in super-block");
+		die(_("bad s_zmap_blocks field in super-block"));
 }
 
-void read_tables(void)
-{
+static void
+read_tables(void) {
 	inode_map = malloc(IMAPS * BLOCK_SIZE);
 	if (!inode_map)
-		die("Unable to allocate buffer for inode map");
+		die(_("Unable to allocate buffer for inode map"));
 	zone_map = malloc(ZMAPS * BLOCK_SIZE);
 	if (!inode_map)
 		die("Unable to allocate buffer for zone map");
@@ -584,37 +598,37 @@ void read_tables(void)
 	memset(zone_map,0,sizeof(zone_map));
 	inode_buffer = malloc(INODE_BUFFER_SIZE);
 	if (!inode_buffer)
-		die("Unable to allocate buffer for inodes");
+		die(_("Unable to allocate buffer for inodes"));
 	inode_count = malloc(INODES + 1);
 	if (!inode_count)
-		die("Unable to allocate buffer for inode count");
+		die(_("Unable to allocate buffer for inode count"));
 	zone_count = malloc(ZONES);
 	if (!zone_count)
-		die("Unable to allocate buffer for zone count");
+		die(_("Unable to allocate buffer for zone count"));
 	if (IMAPS*BLOCK_SIZE != read(IN,inode_map,IMAPS*BLOCK_SIZE))
-		die("Unable to read inode map");
+		die(_("Unable to read inode map"));
 	if (ZMAPS*BLOCK_SIZE != read(IN,zone_map,ZMAPS*BLOCK_SIZE))
-		die("Unable to read zone map");
+		die(_("Unable to read zone map"));
 	if (INODE_BUFFER_SIZE != read(IN,inode_buffer,INODE_BUFFER_SIZE))
-		die("Unable to read inodes");
+		die(_("Unable to read inodes"));
 	if (NORM_FIRSTZONE != FIRSTZONE) {
-		printf("Warning: Firstzone != Norm_firstzone\n");
+		printf(_("Warning: Firstzone != Norm_firstzone\n"));
 		errors_uncorrected = 1;
 	}
 	get_dirsize ();
 	if (show) {
-		printf("%ld inodes\n",INODES);
-		printf("%ld blocks\n",ZONES);
-		printf("Firstdatazone=%ld (%ld)\n",FIRSTZONE,NORM_FIRSTZONE);
-		printf("Zonesize=%d\n",BLOCK_SIZE<<ZONESIZE);
-		printf("Maxsize=%ld\n",MAXSIZE);
-		printf("Filesystem state=%d\n", Super.s_state);
-		printf("namelen=%d\n\n",namelen);
+		printf(_("%ld inodes\n"),INODES);
+		printf(_("%ld blocks\n"),ZONES);
+		printf(_("Firstdatazone=%ld (%ld)\n"),FIRSTZONE,NORM_FIRSTZONE);
+		printf(_("Zonesize=%d\n"),BLOCK_SIZE<<ZONESIZE);
+		printf(_("Maxsize=%ld\n"),MAXSIZE);
+		printf(_("Filesystem state=%d\n"), Super.s_state);
+		printf(_("namelen=%d\n\n"),namelen);
 	}
 }
 
-struct minix_inode * get_inode(unsigned int nr)
-{
+static struct minix_inode *
+get_inode(unsigned int nr) {
 	struct minix_inode * inode;
 
 	if (!nr || nr > INODES)
@@ -623,15 +637,16 @@ struct minix_inode * get_inode(unsigned int nr)
 	inode = Inode + nr;
 	if (!inode_count[nr]) {
 		if (!inode_in_use(nr)) {
-			printf("Inode %d marked not used, but used for file '",
+			printf(_("Inode %d marked not used, but used for file '"),
 				nr);
 			print_current_name();
 			printf("'\n");
-			if (repair)
-				if (ask("Mark in use",1))
+			if (repair) {
+				if (ask(_("Mark in use"),1))
 					mark_inode(nr);
-			else
+			} else {
 			        errors_uncorrected = 1;
+			}
 		}
 		if (S_ISDIR(inode->i_mode))
 			directory++;
@@ -649,13 +664,13 @@ struct minix_inode * get_inode(unsigned int nr)
 			;
 		else {
                         print_current_name();
-                        printf(" has mode %05o\n",inode->i_mode);
+                        printf(_(" has mode %05o\n"),inode->i_mode);
                 }
 
 	} else
 		links++;
 	if (!++inode_count[nr]) {
-		printf("Warning: inode count too big.\n");
+		printf(_("Warning: inode count too big.\n"));
 		inode_count[nr]--;
 		errors_uncorrected = 1;
 	}
@@ -663,9 +678,8 @@ struct minix_inode * get_inode(unsigned int nr)
 }
 
 #ifdef HAVE_MINIX2
-struct minix2_inode *
-get_inode2 (unsigned int nr)
-{
+static struct minix2_inode *
+get_inode2 (unsigned int nr) {
 	struct minix2_inode *inode;
 
 	if (!nr || nr > INODES)
@@ -674,11 +688,11 @@ get_inode2 (unsigned int nr)
 	inode = Inode2 + nr;
 	if (!inode_count[nr]) {
 		if (!inode_in_use (nr)) {
-			printf ("Inode %d marked not used, but used for file '", nr);
+			printf (_("Inode %d marked not used, but used for file '"), nr);
 			print_current_name ();
 			printf ("'\n");
 			if (repair) {
-				if (ask ("Mark in use", 1))
+				if (ask (_("Mark in use"), 1))
 					mark_inode (nr);
 				else
 					errors_uncorrected = 1;
@@ -698,12 +712,12 @@ get_inode2 (unsigned int nr)
 		else if (S_ISFIFO (inode->i_mode));
 		else {
 			print_current_name ();
-			printf (" has mode %05o\n", inode->i_mode);
+			printf (_(" has mode %05o\n"), inode->i_mode);
 		}
 	} else
 		links++;
 	if (!++inode_count[nr]) {
-		printf ("Warning: inode count too big.\n");
+		printf (_("Warning: inode count too big.\n"));
 		inode_count[nr]--;
 		errors_uncorrected = 1;
 	}
@@ -711,17 +725,17 @@ get_inode2 (unsigned int nr)
 }
 #endif
 
-void check_root(void)
-{
+static void
+check_root(void) {
 	struct minix_inode * inode = Inode + ROOT_INO;
 
 	if (!inode || !S_ISDIR(inode->i_mode))
-		die("root inode isn't a directory");
+		die(_("root inode isn't a directory"));
 }
 
 #ifdef HAVE_MINIX2
-void check_root2 (void)
-{
+static void
+check_root2 (void) {
 	struct minix2_inode *inode = Inode2 + ROOT_INO;
 
 	if (!inode || !S_ISDIR (inode->i_mode))
@@ -729,8 +743,8 @@ void check_root2 (void)
 }
 #endif
 
-static int add_zone(unsigned short * znr, int * corrected)
-{
+static int
+add_zone(unsigned short * znr, int * corrected) {
 	int result;
 	int block;
 
@@ -739,10 +753,10 @@ static int add_zone(unsigned short * znr, int * corrected)
 	if (!block)
 		return 0;
 	if (zone_count[block]) {
-		printf("Block has been used before. Now in file `");
+		printf(_("Block has been used before. Now in file `"));
 		print_current_name();
 		printf("'.");
-		if (ask("Clear",1)) {
+		if (ask(_("Clear"),1)) {
 			*znr = 0;
 			block = 0;
 			*corrected = 1;
@@ -751,10 +765,10 @@ static int add_zone(unsigned short * znr, int * corrected)
 	if (!block)
 		return 0;
 	if (!zone_in_use(block)) {
-		printf("Block %d in file `",block);
+		printf(_("Block %d in file `"),block);
 		print_current_name();
-		printf("' is marked not in use.");
-		if (ask("Correct",1))
+		printf(_("' is marked not in use."));
+		if (ask(_("Correct"),1))
 			mark_zone(block);
 	}
 	if (!++zone_count[block])
@@ -762,9 +776,9 @@ static int add_zone(unsigned short * znr, int * corrected)
 	return block;
 }
 
-
-static int add_zone2 (unsigned int *znr, int *corrected)
-{
+#ifdef HAVE_MINIX2
+static int
+add_zone2 (unsigned int *znr, int *corrected) {
 	int result;
 	int block;
 
@@ -773,10 +787,10 @@ static int add_zone2 (unsigned int *znr, int *corrected)
 	if (!block)
 		return 0;
 	if (zone_count[block]) {
-		printf ("Block has been used before. Now in file `");
+		printf (_("Block has been used before. Now in file `"));
 		print_current_name ();
 		printf ("'.");
-		if (ask ("Clear", 1)) {
+		if (ask (_("Clear"), 1)) {
 			*znr = 0;
 			block = 0;
 			*corrected = 1;
@@ -785,19 +799,20 @@ static int add_zone2 (unsigned int *znr, int *corrected)
 	if (!block)
 		return 0;
 	if (!zone_in_use (block)) {
-		printf ("Block %d in file `", block);
+		printf (_("Block %d in file `"), block);
 		print_current_name ();
-		printf ("' is marked not in use.");
-		if (ask ("Correct", 1))
+		printf (_("' is marked not in use."));
+		if (ask (_("Correct"), 1))
 			mark_zone (block);
 	}
 	if (!++zone_count[block])
 		zone_count[block]--;
 	return block;
 }
+#endif
 
-static void add_zone_ind(unsigned short * znr, int * corrected)
-{
+static void
+add_zone_ind(unsigned short * znr, int * corrected) {
 	static char blk[BLOCK_SIZE];
 	int i, chg_blk=0;
 	int block;
@@ -812,8 +827,9 @@ static void add_zone_ind(unsigned short * znr, int * corrected)
 		write_block(block, blk);
 }
 
-static void add_zone_ind2 (unsigned int *znr, int *corrected)
-{
+#ifdef HAVE_MINIX2
+static void
+add_zone_ind2 (unsigned int *znr, int *corrected) {
 	static char blk[BLOCK_SIZE];
 	int i, chg_blk = 0;
 	int block;
@@ -827,9 +843,10 @@ static void add_zone_ind2 (unsigned int *znr, int *corrected)
 	if (chg_blk)
 		write_block (block, blk);
 }
+#endif
 
-static void add_zone_dind(unsigned short * znr, int * corrected)
-{
+static void
+add_zone_dind(unsigned short * znr, int * corrected) {
 	static char blk[BLOCK_SIZE];
 	int i, blk_chg=0;
 	int block;
@@ -844,9 +861,9 @@ static void add_zone_dind(unsigned short * znr, int * corrected)
 		write_block(block, blk);
 }
 
+#ifdef HAVE_MINIX2
 static void
-add_zone_dind2 (unsigned int *znr, int *corrected)
-{
+add_zone_dind2 (unsigned int *znr, int *corrected) {
 	static char blk[BLOCK_SIZE];
 	int i, blk_chg = 0;
 	int block;
@@ -862,8 +879,7 @@ add_zone_dind2 (unsigned int *znr, int *corrected)
 }
 
 static void
-add_zone_tind2 (unsigned int *znr, int *corrected)
-{
+add_zone_tind2 (unsigned int *znr, int *corrected) {
 	static char blk[BLOCK_SIZE];
 	int i, blk_chg = 0;
 	int block;
@@ -877,9 +893,10 @@ add_zone_tind2 (unsigned int *znr, int *corrected)
 	if (blk_chg)
 		write_block (block, blk);
 }
+#endif
 
-void check_zones(unsigned int i)
-{
+static void
+check_zones(unsigned int i) {
 	struct minix_inode * inode;
 
 	if (!i || i > INODES)
@@ -897,9 +914,8 @@ void check_zones(unsigned int i)
 }
 
 #ifdef HAVE_MINIX2
-void
-check_zones2 (unsigned int i)
-{
+static void
+check_zones2 (unsigned int i) {
 	struct minix2_inode *inode;
 
 	if (!i || i > INODES)
@@ -918,8 +934,8 @@ check_zones2 (unsigned int i)
 }
 #endif
 
-void check_file(struct minix_inode * dir, unsigned int offset)
-{
+static void
+check_file(struct minix_inode * dir, unsigned int offset) {
 	static char blk[BLOCK_SIZE];
 	struct minix_inode * inode;
 	int ino;
@@ -932,9 +948,9 @@ void check_file(struct minix_inode * dir, unsigned int offset)
 	ino = * (unsigned short *) (name-2);
 	if (ino > INODES) {
 		print_current_name();
-		printf(" contains a bad inode number for file '");
+		printf(_(" contains a bad inode number for file '"));
 		printf("%.*s'.",namelen,name);
-		if (ask(" Remove",1)) {
+		if (ask(_(" Remove"),1)) {
 			*(unsigned short *)(name-2) = 0;
 			write_block(block, blk);
 		}
@@ -945,22 +961,24 @@ void check_file(struct minix_inode * dir, unsigned int offset)
 	name_depth++;
 	inode = get_inode(ino);
 	name_depth--;
-	if (!offset)
+	if (!offset) {
 		if (!inode || strcmp(".",name)) {
 			print_current_name();
-			printf(": bad directory: '.' isn't first\n");
+			printf(_(": bad directory: '.' isn't first\n"));
 			errors_uncorrected = 1;
 		} else return;
-	if (offset == dirsize)
+	}
+	if (offset == dirsize) {
 		if (!inode || strcmp("..",name)) {
 			print_current_name();
-			printf(": bad directory: '..' isn't second\n");
+			printf(_(": bad directory: '..' isn't second\n"));
 			errors_uncorrected = 1;
 		} else return;
+	}
 	if (!inode)
 		return;
 	if (name_depth < MAX_DEPTH)
-		strncpy(name_list[name_depth],name,namelen);
+		strncpy(name_list[name_depth], name, namelen);
 	name_depth++;	
 	if (list) {
 		if (verbose)
@@ -979,9 +997,8 @@ void check_file(struct minix_inode * dir, unsigned int offset)
 }
 
 #ifdef HAVE_MINIX2
-void
-check_file2 (struct minix2_inode *dir, unsigned int offset)
-{
+static void
+check_file2 (struct minix2_inode *dir, unsigned int offset) {
 	static char blk[BLOCK_SIZE];
 	struct minix2_inode *inode;
 	int ino;
@@ -994,9 +1011,9 @@ check_file2 (struct minix2_inode *dir, unsigned int offset)
 	ino = *(unsigned short *) (name - 2);
 	if (ino > INODES) {
 		print_current_name ();
-		printf (" contains a bad inode number for file '");
+		printf (_(" contains a bad inode number for file '"));
 		printf ("%.*s'.", namelen, name);
-		if (ask (" Remove", 1)) {
+		if (ask (_(" Remove"), 1)) {
 			*(unsigned short *) (name - 2) = 0;
 			write_block (block, blk);
 		}
@@ -1010,7 +1027,7 @@ check_file2 (struct minix2_inode *dir, unsigned int offset)
 	if (!offset) {
 		if (!inode || strcmp (".", name)) {
 			print_current_name ();
-			printf (": bad directory: '.' isn't first\n");
+			printf (_(": bad directory: '.' isn't first\n"));
 			errors_uncorrected = 1;
 		} else
 			return;
@@ -1018,7 +1035,7 @@ check_file2 (struct minix2_inode *dir, unsigned int offset)
 	if (offset == dirsize) {
 		if (!inode || strcmp ("..", name)) {
 			print_current_name ();
-			printf (": bad directory: '..' isn't second\n");
+			printf (_(": bad directory: '..' isn't second\n"));
 			errors_uncorrected = 1;
 		} else
 			return;
@@ -1043,17 +1060,17 @@ check_file2 (struct minix2_inode *dir, unsigned int offset)
 }
 #endif
 
-void recursive_check(unsigned int ino)
-{
+static void
+recursive_check(unsigned int ino) {
 	struct minix_inode * dir;
 	unsigned int offset;
 
 	dir = Inode + ino;
 	if (!S_ISDIR(dir->i_mode))
-		die("internal error");
+		die(_("internal error"));
 	if (dir->i_size < 2 * dirsize) {
 		print_current_name();
-		printf(": bad directory: size<32");
+		printf(_(": bad directory: size<32"));
 		errors_uncorrected = 1;
 	}
 	for (offset = 0 ; offset < dir->i_size ; offset += dirsize)
@@ -1061,9 +1078,8 @@ void recursive_check(unsigned int ino)
 }
 
 #ifdef HAVE_MINIX2
-void
-recursive_check2 (unsigned int ino)
-{
+static void
+recursive_check2 (unsigned int ino) {
 	struct minix2_inode *dir;
 	unsigned int offset;
 
@@ -1072,7 +1088,7 @@ recursive_check2 (unsigned int ino)
 		die ("internal error");
 	if (dir->i_size < 2 * dirsize) {
 		print_current_name ();
-		printf (": bad directory: size < 32");
+		printf (_(": bad directory: size < 32"));
 		errors_uncorrected = 1;
 	}
 	for (offset = 0; offset < dir->i_size; offset += dirsize)
@@ -1080,23 +1096,23 @@ recursive_check2 (unsigned int ino)
 }
 #endif
 
-int bad_zone(int i)
-{
+static int
+bad_zone(int i) {
 	char buffer[1024];
 
 	if (BLOCK_SIZE*i != lseek(IN, BLOCK_SIZE*i, SEEK_SET))
-		die("seek failed in bad_zone");
+		die(_("seek failed in bad_zone"));
 	return (BLOCK_SIZE != read(IN, buffer, BLOCK_SIZE));
 }
 
-void check_counts(void)
-{
+static void
+check_counts(void) {
 	int i;
 
 	for (i=1 ; i <= INODES ; i++) {
 		if (!inode_in_use(i) && Inode[i].i_mode && warn_mode) {
-			printf("Inode %d mode not cleared.",i);
-			if (ask("Clear",1)) {
+			printf(_("Inode %d mode not cleared."),i);
+			if (ask(_("Clear"),1)) {
 				Inode[i].i_mode = 0;
 				changed = 1;
 			}
@@ -1104,21 +1120,21 @@ void check_counts(void)
 		if (!inode_count[i]) {
 			if (!inode_in_use(i))
 				continue;
-			printf("Inode %d not used, marked used in the bitmap.",i);
-			if (ask("Clear",1))
+			printf(_("Inode %d not used, marked used in the bitmap."),i);
+			if (ask(_("Clear"),1))
 				unmark_inode(i);
 			continue;
 		}
 		if (!inode_in_use(i)) {
-			printf("Inode %d used, marked unused in the bitmap.",
+			printf(_("Inode %d used, marked unused in the bitmap."),
 				i);
 			if (ask("Set",1))
 				mark_inode(i);
 		}
 		if (Inode[i].i_nlinks != inode_count[i]) {
-			printf("Inode %d (mode = %07o), i_nlinks=%d, counted=%d.",
+			printf(_("Inode %d (mode = %07o), i_nlinks=%d, counted=%d."),
 				i,Inode[i].i_mode,Inode[i].i_nlinks,inode_count[i]);
-			if (ask("Set i_nlinks to count",1)) {
+			if (ask(_("Set i_nlinks to count"),1)) {
 				Inode[i].i_nlinks=inode_count[i];
 				changed=1;
 			}
@@ -1130,26 +1146,25 @@ void check_counts(void)
 		if (!zone_count[i]) {
 			if (bad_zone(i))
 				continue;
-			printf("Zone %d: marked in use, no file uses it.",i);
-			if (ask("Unmark",1))
+			printf(_("Zone %d: marked in use, no file uses it."),i);
+			if (ask(_("Unmark"),1))
 				unmark_zone(i);
 			continue;
 		}
-		printf("Zone %d: %sin use, counted=%d\n",
-		i,zone_in_use(i)?"":"not ",zone_count[i]);
+		printf(_("Zone %d: %sin use, counted=%d\n"),
+		i,zone_in_use(i)?"":_("not "),zone_count[i]);
 	}
 }
 
 #ifdef HAVE_MINIX2
-void
-check_counts2 (void)
-{
+static void
+check_counts2 (void) {
 	int i;
 
 	for (i = 1; i <= INODES; i++) {
 		if (!inode_in_use (i) && Inode2[i].i_mode && warn_mode) {
-			printf ("Inode %d mode not cleared.", i);
-			if (ask ("Clear", 1)) {
+			printf (_("Inode %d mode not cleared."), i);
+			if (ask (_("Clear"), 1)) {
 				Inode2[i].i_mode = 0;
 				changed = 1;
 			}
@@ -1157,20 +1172,20 @@ check_counts2 (void)
 		if (!inode_count[i]) {
 			if (!inode_in_use (i))
 				continue;
-			printf ("Inode %d not used, marked used in the bitmap.", i);
-			if (ask ("Clear", 1))
+			printf (_("Inode %d not used, marked used in the bitmap."), i);
+			if (ask (_("Clear"), 1))
 				unmark_inode (i);
 			continue;
 		}
 		if (!inode_in_use (i)) {
-			printf ("Inode %d used, marked unused in the bitmap.", i);
-			if (ask ("Set", 1))
+			printf (_("Inode %d used, marked unused in the bitmap."), i);
+			if (ask (_("Set"), 1))
 				mark_inode (i);
 		}
 		if (Inode2[i].i_nlinks != inode_count[i]) {
-			printf ("Inode %d (mode = %07o), i_nlinks=%d, counted=%d.",
+			printf (_("Inode %d (mode = %07o), i_nlinks=%d, counted=%d."),
 				i, Inode2[i].i_mode, Inode2[i].i_nlinks, inode_count[i]);
-			if (ask ("Set i_nlinks to count", 1)) {
+			if (ask (_("Set i_nlinks to count"), 1)) {
 				Inode2[i].i_nlinks = inode_count[i];
 				changed = 1;
 			}
@@ -1182,19 +1197,19 @@ check_counts2 (void)
 		if (!zone_count[i]) {
 			if (bad_zone (i))
 				continue;
-			printf ("Zone %d: marked in use, no file uses it.", i);
-			if (ask ("Unmark", 1))
+			printf (_("Zone %d: marked in use, no file uses it."), i);
+			if (ask (_("Unmark"), 1))
 				unmark_zone (i);
 			continue;
 		}
-		printf ("Zone %d: %sin use, counted=%d\n",
-			i, zone_in_use (i) ? "" : "not ", zone_count[i]);
+		printf (_("Zone %d: %sin use, counted=%d\n"),
+			i, zone_in_use (i) ? "" : _("not "), zone_count[i]);
 	}
 }
 #endif
 
-void check(void)
-{
+static void
+check(void) {
 	memset(inode_count,0,(INODES + 1) * sizeof(*inode_count));
 	memset(zone_count,0,ZONES*sizeof(*zone_count));
 	check_zones(ROOT_INO);
@@ -1203,9 +1218,8 @@ void check(void)
 }
 
 #ifdef HAVE_MINIX2
-void
-check2 (void)
-{
+static void
+check2 (void) {
 	memset (inode_count, 0, (INODES + 1) * sizeof (*inode_count));
 	memset (zone_count, 0, ZONES * sizeof (*zone_count));
 	check_zones2 (ROOT_INO);
@@ -1214,28 +1228,41 @@ check2 (void)
 }
 #endif
 
-int main(int argc, char ** argv)
-{
+int
+main(int argc, char ** argv) {
 	struct termios tmp;
 	int count;
 	int retcode = 0;
+	char *p;
+
+	program_name = (argc && *argv) ? argv[0] : "fsck.minix";
+	if ((p = strrchr(program_name, '/')) != NULL)
+		program_name = p+1;
+
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+
+	if (argc == 2 &&
+	    (!strcmp(argv[1], "-V") || !strcmp(argv[1], "--version"))) {
+		printf(_("%s from %s\n"), program_name, util_linux_version);
+		exit(0);
+	}
 
-	if (argc && *argv)
-		program_name = *argv;
 	if (INODE_SIZE * MINIX_INODES_PER_BLOCK != BLOCK_SIZE)
-		die("bad inode size");
+		die(_("bad inode size"));
 #ifdef HAVE_MINIX2
 	if (INODE_SIZE2 * MINIX2_INODES_PER_BLOCK != BLOCK_SIZE)
-		die("bad v2 inode size");
+		die(_("bad v2 inode size"));
 #endif
 	while (argc-- > 1) {
 		argv++;
-		if (argv[0][0] != '-')
+		if (argv[0][0] != '-') {
 			if (device_name)
 				usage();
 			else
 				device_name = argv[0];
-		else while (*++argv[0])
+		} else while (*++argv[0])
 			switch (argv[0][0]) {
 				case 'l': list=1; break;
 				case 'a': automatic=1; repair=1; break;
@@ -1252,11 +1279,11 @@ int main(int argc, char ** argv)
 	check_mount();		/* trying to check a mounted filesystem? */
 	if (repair && !automatic) {
 		if (!isatty(0) || !isatty(1))
-			die("need terminal for interactive repairs");
+			die(_("need terminal for interactive repairs"));
 	}
 	IN = open(device_name,repair?O_RDWR:O_RDONLY);
 	if (IN < 0)
-		die("unable to open '%s'");
+		die(_("unable to open '%s'"));
 	for (count=0 ; count<3 ; count++)
 		sync();
 	read_superblock();
@@ -1267,19 +1294,17 @@ int main(int argc, char ** argv)
 	 * flags and whether or not the -f switch was specified on the 
 	 * command line.
 	 */
-	printf("%s, %s / %s\n", program_name, program_version,
-	       util_linux_version);
 	if ( !(Super.s_state & MINIX_ERROR_FS) && 
 	      (Super.s_state & MINIX_VALID_FS) && 
 	      !force ) {
 		if (repair)
-			printf("%s is clean, no check.\n", device_name);
+			printf(_("%s is clean, no check.\n"), device_name);
 		return retcode;
 	}
 	else if (force)
-		printf("Forcing filesystem check on %s.\n", device_name);
+		printf(_("Forcing filesystem check on %s.\n"), device_name);
 	else if (repair)
-		printf("Filesystem on %s is dirty, needs checking.\n",\
+		printf(_("Filesystem on %s is dirty, needs checking.\n"),\
 			device_name);
 
 	read_tables();
@@ -1308,29 +1333,29 @@ int main(int argc, char ** argv)
 		for (i=1,free=0 ; i <= INODES ; i++)
 			if (!inode_in_use(i))
 				free++;
-		printf("\n%6ld inodes used (%ld%%)\n",(INODES-free),
+		printf(_("\n%6ld inodes used (%ld%%)\n"),(INODES-free),
 			100*(INODES-free)/INODES);
 		for (i=FIRSTZONE,free=0 ; i < ZONES ; i++)
 			if (!zone_in_use(i))
 				free++;
-		printf("%6ld zones used (%ld%%)\n",(ZONES-free),
+		printf(_("%6ld zones used (%ld%%)\n"),(ZONES-free),
 			100*(ZONES-free)/ZONES);
-		printf("\n%6d regular files\n"
+		printf(_("\n%6d regular files\n"
 		"%6d directories\n"
 		"%6d character device files\n"
 		"%6d block device files\n"
 		"%6d links\n"
 		"%6d symbolic links\n"
 		"------\n"
-		"%6d files\n",
+		"%6d files\n"),
 		regular,directory,chardev,blockdev,
 		links-2*directory+1,symlinks,total-2*directory+1);
 	}
 	if (changed) {
 		write_tables();
-		printf(	"----------------------------\n"
+		printf(_(	"----------------------------\n"
 			"FILE SYSTEM HAS BEEN CHANGED\n"
-			"----------------------------\n");
+			"----------------------------\n"));
 		for (count=0 ; count<3 ; count++)
 			sync();
 	}
