@@ -4,6 +4,13 @@
    be here. Now read the real man-page agetty.8 instead.
 
    -f option added by Eric Rasmussen <ear@usfirst.org> - 12/28/95
+   
+   1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+   - added Native Language Support
+
+   1999-05-05 Thorsten Kranzkowski <dl8bcu@gmx.net>
+   - enable hardware flow control before displaying /etc/issue
+   
 */
 
 #include <stdio.h>
@@ -17,12 +24,13 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <varargs.h>
+#include <stdarg.h>
 #include <ctype.h>
 #include <utmp.h>
 #include <getopt.h>
-#include <memory.h>
+#include <time.h>
 #include <sys/file.h>
+#include "nls.h"
 
 #ifdef __linux__
 #include "pathnames.h"
@@ -34,7 +42,6 @@
 
 #ifdef	USE_SYSLOG
 #include <syslog.h>
-extern void closelog();
 #endif
 
  /*
@@ -208,12 +215,13 @@ static struct Speedtab speedtab[] = {
     { 0, 0 },
 };
 
-#define P_(s) ()
+#define P_(s) s
+int main P_((int argc, char **argv));
 void parse_args P_((int argc, char **argv, struct options *op));
 void parse_speeds P_((struct options *op, char *arg));
 void update_utmp P_((char *line));
 void open_tty P_((char *tty, struct termio *tp, int local));
-void termio_init P_((struct termio *tp, int speed, int local));
+void termio_init P_((struct termio *tp, int speed, struct options *op));
 void auto_baud P_((struct termio *tp));
 void do_prompt P_((struct options *op, struct termio *tp));
 void next_speed P_((struct termio *tp, struct options *op));
@@ -222,13 +230,16 @@ void termio_final P_((struct options *op, struct termio *tp, struct chardata *cp
 int caps_lock P_((char *s));
 int bcode P_((char *s));
 void usage P_((void));
-void error P_((int va_alist));
+void error P_((const char *, ...));
 #undef P_
 
 /* The following is used for understandable diagnostics. */
 
 char *progname;
 
+/* Fake hostname for ut_host specified on command line. */
+char *fakehost = NULL;
+
 /* ... */
 #ifdef DEBUGGING
 #define debug(s) fprintf(dbf,s); fflush(dbf)
@@ -255,6 +266,10 @@ main(argc, argv)
 	0,				/* no baud rates known yet */
     };
 
+       setlocale(LC_ALL, "");
+       bindtextdomain(PACKAGE, LOCALEDIR);
+       textdomain(PACKAGE);
+    
     /* The BSD-style init command passes us a useless process name. */
 
 #ifdef	SYSV_STYLE
@@ -288,7 +303,7 @@ main(argc, argv)
     update_utmp(options.tty);
 #endif
 
-    debug("calling open_tty\n");
+    debug(_("calling open_tty\n"));
     /* Open the tty as standard { input, output, error }. */
     open_tty(options.tty, &termio, options.flags & F_LOCAL);
 
@@ -301,12 +316,12 @@ main(argc, argv)
 	}
 #endif
     /* Initialize the termio settings (raw mode, eight-bit, blocking i/o). */
-    debug("calling termio_init\n");
-    termio_init(&termio, options.speeds[FIRST_SPEED], options.flags & F_LOCAL);
+    debug(_("calling termio_init\n"));
+    termio_init(&termio, options.speeds[FIRST_SPEED], &options);
 
     /* write the modem init string and DON'T flush the buffers */
     if (options.flags & F_INITSTRING) {
-	debug("writing init string\n");
+	debug(_("writing init string\n"));
 	write(1, options.initstring, strlen(options.initstring));
     }
 
@@ -316,7 +331,7 @@ main(argc, argv)
     }
 
     /* Optionally detect the baud rate from the modem status message. */
-    debug("before autobaud\n");
+    debug(_("before autobaud\n"));
     if (options.flags & F_PARSE)
 	auto_baud(&termio);
 
@@ -328,11 +343,11 @@ main(argc, argv)
     if (options.flags & F_WAITCRLF) {
 	char ch;
 
-	debug("waiting for cr-lf\n");
+	debug(_("waiting for cr-lf\n"));
 	while(read(0, &ch, 1) == 1) {
 	    ch &= 0x7f;   /* strip "parity bit" */
 #ifdef DEBUGGING
-	    fprintf(dbf, "read %c\n", ch);
+	    fprintf(dbf, _("read %c\n"), ch);
 #endif
 	    if (ch == '\n' || ch == '\r') break;
 	}
@@ -341,7 +356,7 @@ main(argc, argv)
     chardata = init_chardata;
     if (!(options.flags & F_NOPROMPT)) {
 	/* Read the login name. */
-	debug("reading login name\n");
+	debug(_("reading login name\n"));
 	while ((logname = get_logname(&options, &chardata, &termio)) == 0)
 	  next_speed(&termio, &options);
     }
@@ -362,7 +377,7 @@ main(argc, argv)
     /* Let the login program take care of password validation. */
 
     (void) execl(options.login, options.login, "--", logname, (char *) 0);
-    error("%s: can't exec %s: %m", options.tty, options.login);
+    error(_("%s: can't exec %s: %m"), options.tty, options.login);
     exit(0);  /* quiet GCC */
 }
 
@@ -378,11 +393,11 @@ parse_args(argc, argv, op)
     extern int optind;			/* getopt */
     int     c;
 
-    while (isascii(c = getopt(argc, argv, "I:Lf:hil:mt:wn"))) {
+    while (isascii(c = getopt(argc, argv, "I:LH:f:hil:mt:wn"))) {
 	switch (c) {
 	case 'I':
 	    if (!(op->initstring = malloc(strlen(optarg)))) {
-		error("can't malloc initstring");
+		error(_("can't malloc initstring"));
 		break;
 	    }
 	    {
@@ -423,6 +438,9 @@ parse_args(argc, argv, op)
 	case 'L':				/* force local */
  	    op->flags |= F_LOCAL;
  	    break;
+	case 'H':                               /* fake login host */
+	    fakehost = optarg;
+	    break;
 	case 'f':				/* custom issue file */
 	    op->flags |= F_CUSTISSUE;
 	    op->issue = optarg;
@@ -444,7 +462,7 @@ parse_args(argc, argv, op)
 	    break;
 	case 't':				/* time out */
 	    if ((op->timeout = atoi(optarg)) <= 0)
-		error("bad timeout value: %s", optarg);
+		error(_("bad timeout value: %s"), optarg);
 	    break;
 	case 'w':
 	    op->flags |= F_WAITCRLF;
@@ -453,7 +471,7 @@ parse_args(argc, argv, op)
 	    usage();
 	}
     }
- 	debug("after getopt loop\n");
+ 	debug(_("after getopt loop\n"));
     if (argc < optind + 2)			/* check parameter count */
 	usage();
 
@@ -471,7 +489,39 @@ parse_args(argc, argv, op)
     if (argc > optind && argv[optind])
 	setenv ("TERM", argv[optind], 1);
 
-    debug("exiting parseargs\n");
+#ifdef DO_DEVFS_FIDDLING
+    /*
+     * some devfs junk, following Goswin Brederlow:
+     *   turn ttyS<n> into tts/<n>
+     *   turn tty<n> into vc/<n>
+     */
+    if (op->tty && strlen(op->tty) < 90) {
+	    char dev_name[100];
+	    struct stat st;
+
+	    if (strncmp(op->tty, "ttyS", 4) == 0) {
+		    strcpy(dev_name, "/dev/");
+		    strcat(dev_name, op->tty);
+		    if (stat(dev_name, &st) < 0) {
+			    strcpy(dev_name, "/dev/tts/");
+			    strcat(dev_name, op->tty + 4);
+			    if (stat(dev_name, &st) == 0)
+				    op->tty = strdup(dev_name + 5);
+		    }
+	    } else if (strncmp(op->tty, "tty", 3) == 0) {
+		    strcpy(dev_name, "/dev/");
+		    strncat(dev_name, op->tty, 90);
+		    if (stat(dev_name, &st) < 0) {
+			    strcpy(dev_name, "/dev/vc/");
+			    strcat(dev_name, op->tty + 3);
+			    if (stat(dev_name, &st) == 0)
+				    op->tty = strdup(dev_name + 5);
+		    }
+	    }
+    }
+#endif
+
+    debug(_("exiting parseargs\n"));
 }
 
 /* parse_speeds - parse alternate baud rates */
@@ -481,17 +531,16 @@ parse_speeds(op, arg)
      struct options *op;
      char   *arg;
 {
-    char   *strtok();
     char   *cp;
 
-	debug("entered parse_speeds\n");
+	debug(_("entered parse_speeds\n"));
     for (cp = strtok(arg, ","); cp != 0; cp = strtok((char *) 0, ",")) {
 	if ((op->speeds[op->numspeed++] = bcode(cp)) <= 0)
-	    error("bad speed: %s", cp);
-	if (op->numspeed > MAX_SPEED)
-	    error("too many alternate speeds");
+	    error(_("bad speed: %s"), cp);
+	if (op->numspeed >= MAX_SPEED)
+	    error(_("too many alternate speeds"));
     }
-    debug("exiting parsespeeds\n");
+    debug(_("exiting parsespeeds\n"));
 }
 
 #ifdef	SYSV_STYLE
@@ -503,10 +552,7 @@ update_utmp(line)
 {
     struct  utmp ut;
     time_t  t;
-    int     ut_fd;
     int     mypid = getpid();
-    long    time();
-    long    lseek();
     struct  utmp *utp;
 
     /*
@@ -518,7 +564,6 @@ update_utmp(line)
      * entry in the utmp file.
      */
 
-#ifdef __linux__
     utmpname(_PATH_UTMP);
     setutent();
     while ((utp = getutent()) 
@@ -532,10 +577,11 @@ update_utmp(line)
 	memset(&ut, 0, sizeof(ut));
 	strncpy(ut.ut_id, line + 3, sizeof(ut.ut_id));
     }
-    /*endutent();*/
 	
     strncpy(ut.ut_user, "LOGIN", sizeof(ut.ut_user));
     strncpy(ut.ut_line, line, sizeof(ut.ut_line));
+    if (fakehost)
+	strncpy(ut.ut_host, fakehost, sizeof(ut.ut_host));
     time(&t);
     ut.ut_time = t;
     ut.ut_type = LOGIN_PROCESS;
@@ -545,6 +591,10 @@ update_utmp(line)
     endutent();
 
     {
+#ifdef HAVE_updwtmp
+	updwtmp(_PATH_WTMP, &ut);
+#else
+	int ut_fd;
 	int lf;
 
 	if ((lf = open(_PATH_WTMPLOCK, O_CREAT|O_WRONLY, 0660)) >= 0) {
@@ -556,28 +606,8 @@ update_utmp(line)
 	    flock(lf, LOCK_UN);
 	    close(lf);
 	}
+#endif
     }
-#else
-    if ((ut_fd = open(UTMP_FILE, 2)) < 0) {
-	error("%s: open for update: %m", UTMP_FILE);
-    } else {
-	long    ut_size = sizeof(ut);       /* avoid nonsense */
-
-	while (read(ut_fd, (char *) &ut, sizeof(ut)) == sizeof(ut)) {
-	    if (ut.ut_type == INIT_PROCESS && ut.ut_pid == mypid) {
-		ut.ut_type = LOGIN_PROCESS;
-		ut.ut_time = time((long *) 0);
-		(void) strncpy(ut.ut_name, "LOGIN", sizeof(ut.ut_name));
-		(void) strncpy(ut.ut_line, line, sizeof(ut.ut_line));
-		(void) lseek(ut_fd, -ut_size, 1);
-		(void) write(ut_fd, (char *) &ut, sizeof(ut));
-		(void) close(ut_fd);
-		return;
-	    }
-	}
-	error("%s: no utmp entry", line);
-    }
-#endif /* __linux__ */
 }
 
 #endif
@@ -603,20 +633,20 @@ open_tty(tty, tp, local)
 	/* Sanity checks... */
 
 	if (chdir("/dev"))
-	    error("/dev: chdir() failed: %m");
+	    error(_("/dev: chdir() failed: %m"));
 	if (stat(tty, &st) < 0)
 	    error("/dev/%s: %m", tty);
 	if ((st.st_mode & S_IFMT) != S_IFCHR)
-	    error("/dev/%s: not a character device", tty);
+	    error(_("/dev/%s: not a character device"), tty);
 
 	/* Open the tty as standard input. */
 
 	(void) close(0);
 	errno = 0;				/* ignore close(2) errors */
 
-	debug("open(2)\n");
+	debug(_("open(2)\n"));
 	if (open(tty, O_RDWR|O_NONBLOCK, 0) != 0)
-	    error("/dev/%s: cannot open as standard input: %m", tty);
+	    error(_("/dev/%s: cannot open as standard input: %m"), tty);
 
     } else {
 
@@ -626,13 +656,13 @@ open_tty(tty, tp, local)
 	 */
 
 	if ((fcntl(0, F_GETFL, 0) & O_RDWR) != O_RDWR)
-	    error("%s: not open for read/write", tty);
+	    error(_("%s: not open for read/write"), tty);
     }
 
     /* Set up standard output and standard error file descriptors. */
-    debug("duping\n");
+    debug(_("duping\n"));
     if (dup(0) != 1 || dup(0) != 2)		/* set up stdout and stderr */
-	error("%s: dup problem: %m", tty);	/* we have a problem */
+	error(_("%s: dup problem: %m"), tty);	/* we have a problem */
 
     /*
      * The following ioctl will fail if stdin is not a tty, but also when
@@ -664,10 +694,10 @@ char gbuf[1024];
 char area[1024];
 
 void
-termio_init(tp, speed, local)
+termio_init(tp, speed, op)
      struct termio *tp;
      int     speed;
-     int	local;
+     struct options *op;
 {
 
     /*
@@ -682,19 +712,27 @@ termio_init(tp, speed, local)
 #endif
 
     tp->c_cflag = CS8 | HUPCL | CREAD | speed;
-    if (local) {
+    if (op->flags & F_LOCAL) {
 	tp->c_cflag |= CLOCAL;
     }
 
     tp->c_iflag = tp->c_lflag = tp->c_oflag = tp->c_line = 0;
     tp->c_cc[VMIN] = 1;
     tp->c_cc[VTIME] = 0;
+
+    /* Optionally enable hardware flow control */
+
+#ifdef	CRTSCTS
+    if (op->flags & F_RTSCTS)
+	tp->c_cflag |= CRTSCTS;
+#endif
+
     (void) ioctl(0, TCSETA, tp);
 
     /* go to blocking input even in local mode */
     fcntl(0, F_SETFL, fcntl(0, F_GETFL, 0) & ~O_NONBLOCK);
 
-    debug("term_io 2\n");
+    debug(_("term_io 2\n"));
 }
 
 /* auto_baud - extract baud rate from modem status message */
@@ -769,7 +807,7 @@ do_prompt(op, tp)
 #ifdef	ISSUE
     FILE    *fd;
     int     oflag;
-    char    c;
+    int     c;
     struct utsname uts;
 
     (void) uname(&uts);
@@ -814,8 +852,11 @@ do_prompt(op, tp)
 		  case 'o':
 		   {
 		     char domainname[256];
-
+#ifdef HAVE_getdomainname
 		     getdomainname(domainname, sizeof(domainname));
+#else
+		     strcpy(domainname, "unknown_domain");
+#endif
 		     domainname[sizeof(domainname)-1] = '\0';
 		     printf ("%s", domainname);
 		   }
@@ -874,9 +915,9 @@ do_prompt(op, tp)
 		        if (ut->ut_type == USER_PROCESS)
 			  users++;
 		      endutent();
-		      printf ("%d", users);
+		      printf ("%d ", users);
 		      if (c == 'U')
-		        printf (" user%s", users == 1 ? "" : "s");
+		        printf ((users == 1) ? _("user") : _("users"));
 		      break;
 		    }
 		  default:
@@ -964,7 +1005,7 @@ char   *get_logname(op, cp, tp)
 	    if (read(0, &c, 1) < 1) {
 		if (errno == EINTR || errno == EIO)
 		    exit(0);
-		error("%s: read: %m", op->tty);
+		error(_("%s: read: %m"), op->tty);
 	    }
 	    /* Do BREAK handling elsewhere. */
 
@@ -1010,7 +1051,7 @@ char   *get_logname(op, cp, tp)
 		if (!isascii(ascval) || !isprint(ascval)) {
 		     /* ignore garbage characters */ ;
 		} else if (bp - logname >= sizeof(logname) - 1) {
-		    error("%s: input overrun", op->tty);
+		    error(_("%s: input overrun"), op->tty);
 		} else {
 		    (void) write(1, &c, 1);	/* echo the character */
 		    *bp++ = ascval;		/* and store it */
@@ -1134,10 +1175,7 @@ bcode(s)
 void
 usage()
 {
-    static char msg[] =
-    "[-hiLmw] [-l login_program] [-t timeout] [-I initstring] baud_rate,... line [termtype]\nor\t[-hiLmw] [-l login_program] [-t timeout] [-I initstring] line baud_rate,... [termtype]";
-
-    fprintf(stderr, "Usage: %s %s\n", progname, msg);
+    fprintf(stderr, _("Usage: %s [-hiLmw] [-l login_program] [-t timeout] [-I initstring] [-H login_host] baud_rate,... line [termtype]\nor\t[-hiLmw] [-l login_program] [-t timeout] [-I initstring] [-H login_host] line baud_rate,... [termtype]\n"), progname);
     exit(1);
 }
 
@@ -1145,13 +1183,9 @@ usage()
 
 #define	str2cpy(b,s1,s2)	strcat(strcpy(b,s1),s2)
 
-/* VARARGS */
 void
-error(va_alist)
-     va_dcl
-{
+error(const char *fmt, ...) {
     va_list ap;
-    char   *fmt;
 #ifndef	USE_SYSLOG
     int     fd;
 #endif
@@ -1181,15 +1215,14 @@ error(va_alist)
      * vsprintf() like interface.
      */
 
-    va_start(ap);
-    fmt = va_arg(ap, char *);
+    va_start(ap, fmt);
     while (*fmt) {
 	if (strncmp(fmt, "%s", 2) == 0) {
 	    (void) strcpy(bp, va_arg(ap, char *));
 	    bp += strlen(bp);
 	    fmt += 2;
 	} else if (strncmp(fmt, "%m", 2) == 0) {
-	    (void) strcpy(bp, sys_errlist[errno]);
+	    (void) strcpy(bp, strerror(errno));
 	    bp += strlen(bp);
 	    fmt += 2;
 	} else {
@@ -1216,6 +1249,6 @@ error(va_alist)
 	(void) close(fd);
     }
 #endif
-    (void) sleep((unsigned) 10);			/* be kind to init(8) */
+    (void) sleep((unsigned) 10);		/* be kind to init(8) */
     exit(1);
 }
