@@ -1,6 +1,10 @@
 /* checktty.c - linked into login, checks user against /etc/usertty
    Created 25-Aug-95 by Peter Orbaek <poe@daimi.aau.dk>
    Fixed by JDS June 1996 to clear lists and close files
+
+   1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+   - added Native Language Support
+
 */
 
 #include <sys/types.h>
@@ -17,6 +21,7 @@
 #include <malloc.h>
 #include <netdb.h>
 #include <sys/syslog.h>
+#include "nls.h"
 
 #ifdef __linux__
 #  include <sys/sysmacros.h>
@@ -24,12 +29,8 @@
 #endif
 
 #include "pathnames.h"
-
-/* functions in login.c */
-void badlogin(const char *s);
-void sleepexit(int);
-extern struct hostent hostaddress;
-extern char *hostname;
+#include "login.h"
+#include "xstrncpy.h"
 
 #ifdef TESTING
 struct hostent hostaddress;
@@ -100,15 +101,14 @@ new_class(char *class)
 
     tc = (struct ttyclass *)malloc(sizeof(struct ttyclass));
     if (tc == NULL) {
-	printf("login: memory low, login may fail\n");
-	syslog(LOG_WARNING, "can't malloc for ttyclass");
+	printf(_("login: memory low, login may fail\n"));
+	syslog(LOG_WARNING, _("can't malloc for ttyclass"));
 	return NULL;
     }
 
     tc->next = ttyclasses;
     tc->first = NULL;
-    strncpy(tc->classname, class, CLASSNAMELEN);
-    tc->classname[CLASSNAMELEN-1] = 0;
+    xstrncpy(tc->classname, class, CLASSNAMELEN);
     ttyclasses = tc;
     return tc;
 }
@@ -122,14 +122,13 @@ add_to_class(struct ttyclass *tc, char *tty)
 
     ge = (struct grplist *)malloc(sizeof(struct grplist));
     if (ge == NULL) {
-	printf("login: memory low, login may fail\n");
-	syslog(LOG_WARNING, "can't malloc for grplist");
+	printf(_("login: memory low, login may fail\n"));
+	syslog(LOG_WARNING, _("can't malloc for grplist"));
 	return;
     }
 
     ge->next = tc->first;
-    strncpy(ge->name, tty, NAMELEN);
-    ge->name[NAMELEN-1] = 0;
+    xstrncpy(ge->name, tty, NAMELEN);
     tc->first = ge;
 }
 
@@ -146,20 +145,27 @@ isapty(const char *tty)
 	    return 0;
     sprintf(devname, "/dev/%s", tty);
 
-#if defined(__linux__) && defined(PTY_SLAVE_MAJOR)
-    /* this is for linux 1.3 and newer */
-    if((stat(devname, &stb) >= 0)
-       && major(stb.st_rdev) == PTY_SLAVE_MAJOR) {
-	return 1;
-    }
+#if defined(__linux__)
+    if((stat(devname, &stb) >= 0) && S_ISCHR(stb.st_mode)) {
+	    int majordev = major(stb.st_rdev);
+
+	    /* this is for linux versions before 1.3: use major 4 */
+	    if(majordev == TTY_MAJOR && minor(stb.st_rdev) >= 192)
+		    return 1;
+
+#if defined(PTY_SLAVE_MAJOR)
+	    /* this is for linux 1.3 and newer: use major 3 */
+	    if(majordev == PTY_SLAVE_MAJOR)
+		    return 1;
+#endif
+
+#if defined(UNIX98_PTY_SLAVE_MAJOR) && defined(UNIX98_PTY_MAJOR_COUNT)
+	    /* this is for linux 2.1.116 and newer: use majors 136-143 */
+	    if(majordev >= UNIX98_PTY_SLAVE_MAJOR &&
+	       majordev < UNIX98_PTY_SLAVE_MAJOR + UNIX98_PTY_MAJOR_COUNT)
+		    return 1;
 #endif
 
-#if defined(__linux__)
-    /* this is for linux versions before 1.3, backward compat. */
-    if((stat(devname, &stb) >= 0)
-       && major(stb.st_rdev) == TTY_MAJOR
-       && minor(stb.st_rdev) >= 192) {
-	return 1;
     }
 #endif
     return 0;
@@ -260,8 +266,7 @@ in_class(const char *tty, char *class)
     if (class[0] == '[') {
 	if ((p = strchr(class, ']'))) {
 	    *p = 0;
-	    strncpy(timespec, class+1, sizeof(timespec));
-	    timespec[sizeof(timespec)-1] = 0;
+	    xstrncpy(timespec, class+1, sizeof(timespec));
 	    *p = ']';
 	    if(!timeok(tm, timespec)) return 0;
 	    class = p+1;
@@ -282,8 +287,7 @@ in_class(const char *tty, char *class)
 		if (n[0] == '[') {
 		    if ((p = strchr(n, ']'))) {
 			*p = 0;
-			strncpy(timespec, n+1, sizeof(timespec));
-			timespec[sizeof(timespec)-1] = 0;
+			xstrncpy(timespec, n+1, sizeof(timespec));
 			*p = ']';
 			if(!timeok(tm, timespec)) continue;
 			n = p+1;
@@ -303,7 +307,7 @@ in_class(const char *tty, char *class)
 }
 
 /* start JDS - SBA */
-void 
+static void 
 free_group(struct grplist *ge)
 {
     if (ge) {
@@ -314,7 +318,7 @@ free_group(struct grplist *ge)
     }
 }
 
-void 
+static void 
 free_class(struct ttyclass *tc)
 {
     if (tc) {
@@ -327,7 +331,7 @@ free_class(struct ttyclass *tc)
     }
 }
 
-void 
+static void 
 free_all(void)
 {
     free_class(ttyclasses);
@@ -366,8 +370,7 @@ checktty(const char *user, const char *tty, struct passwd *pwd)
 	  if(*ptr == '#') *ptr = 0;
 
 	if (buf[0] == '*') {
-	    strncpy(defaultbuf, buf, 256);
-	    defaultbuf[255] = 0;
+	    xstrncpy(defaultbuf, buf, 256);
 	    continue;
 	}
 
@@ -416,7 +419,7 @@ checktty(const char *user, const char *tty, struct passwd *pwd)
 	}
 
 	/* there was a default rule, but user didn't match, reject! */
-	printf("Login on %s from %s denied by default.\n", tty, hostname);
+	printf(_("Login on %s from %s denied by default.\n"), tty, hostname);
 	badlogin(user);
 	sleepexit(1);
     }
@@ -427,7 +430,7 @@ checktty(const char *user, const char *tty, struct passwd *pwd)
 	   name of the tty where the user is trying to log in.
 	   So deny access! */
 
-	printf("Login on %s from %s denied.\n", tty, hostname);
+	printf(_("Login on %s from %s denied.\n"), tty, hostname);
 	badlogin(user);
 	sleepexit(1);
     }
