@@ -30,7 +30,7 @@
    
 				    'Lastlog' feature.
    
-   - A lot of nitty gritty details has been adjusted in favour of
+   - A lot of nitty gritty details have been adjusted in favour of
      HP-UX, e.g. /etc/securetty, default paths and the environment
      variables assigned by 'login'.
    
@@ -51,6 +51,11 @@
    1992-02-05 poe@daimi.aau.dk: Ported the stuff to Linux 0.12
    From 1992 till now (1997) this code for Linux has been maintained at
    ftp.daimi.aau.dk:/pub/linux/poe/
+
+   1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+    - added Native Language Support
+   Sun Mar 21 1999 - Arnaldo Carvalho de Melo <acme@conectiva.com.br>
+    - fixed strerr(errno) in gettext calls
  */
 
 /*
@@ -111,7 +116,11 @@
 #include <sys/syslog.h>
 #include <sys/sysmacros.h>
 #include <netdb.h>
+#include "pathnames.h"
 #include "my_crypt.h"
+#include "login.h"
+#include "xstrncpy.h"
+#include "nls.h"
 
 #ifdef __linux__
 #  include <sys/sysmacros.h>
@@ -137,8 +146,11 @@
        syslog(LOG_ERR,"%s",pam_strerror(pamh, retcode)); \
        pam_end(pamh, retcode); exit(1); \
    }
-#  define PAM_END { retcode = pam_close_session(pamh,0); \
-		    pam_end(pamh,retcode); }
+#  define PAM_END { \
+	pam_setcred(pamh, PAM_DELETE_CRED); \
+	retcode = pam_close_session(pamh,0); \
+	pam_end(pamh,retcode); \
+}
 #endif
 
 #ifndef __linux__
@@ -148,6 +160,31 @@
 
 #define SLEEP_EXIT_TIMEOUT 5
 
+#ifdef __linux__
+#define DO_PS_FIDDLING
+#endif
+
+#ifdef DO_PS_FIDDLING
+#include "setproctitle.h"
+#endif
+
+/*
+ * RedHat writes:
+ * we've got a REAL HACK to avoid telling people that they have
+ * mail because the imap server has left a turd in their inbox.
+ * It works, but it sucks...
+ * It turns out that the turd is always 523 bytes long, so we
+ * just check for that size.
+ */
+/*
+ * If you want to turn this strange hack off, set
+	#define REDHAT_IGNORED_MAILSIZE 0
+ * In case people complain, this may become a configuration option,
+ * or perhaps this hack is thrown out again.
+ * A better solution would be to check the contents of this file..
+ */
+#define REDHAT_IGNORED_MAILSIZE	523
+
 #if 0
 /* from before we had a lastlog.h file in linux */
 struct  lastlog
@@ -157,25 +194,23 @@ struct  lastlog
 };
 #endif
 
-#include "pathnames.h"
+#ifndef USE_PAM
+static void getloginname (void);
+static void checknologin (void);
+static int rootterm (char *ttyn);
+#endif
+static void timedout (int);
+static void sigint (int);
+static void motd (void);
+static void dolastlog (int quiet);
 
-#define P_(s) ()
-void opentty P_((const char *tty));
-void getloginname P_((void));
-void timedout P_((void));
-int rootterm P_((char *ttyn));
-void motd P_((void));
-void sigint P_((void));
-void checknologin P_((void));
-void dolastlog P_((int quiet));
-void badlogin P_((const char *name));
-char *stypeof P_((char *ttyid));
-void checktty P_((char *user, char *tty, struct passwd *pwd));
-void sleepexit P_((int eval));
+#ifndef __linux__
+static char *stypeof (char *ttyid);
+#endif
+
 #ifdef CRYPTOCARD
-int cryptocard P_((void));
+#include "cryptocard.h"
 #endif
-#undef P_
 
 #ifdef	KERBEROS
 #include <kerberos/krb.h>
@@ -204,14 +239,14 @@ int	kerror = KSUCCESS, notickets = 1;
 #ifndef __linux__
 int	timeout = 300;
 #else
-int     timeout = 60;
+int     timeout = 60;		/* used in cryptocard.c */
 #endif
 
-struct	passwd *pwd;
-int	failures = 1;
+struct	passwd *pwd;		/* used in cryptocard.c */
+struct  hostent hostaddress;	/* used in checktty.c */
 char	term[64], *hostname, *username, *tty;
-struct  hostent hostaddress;
-char	thishost[100];
+static char	thishost[100];
+static int	failures = 1;
 
 #ifndef __linux__
 struct	sgttyb sgttyb;
@@ -227,12 +262,23 @@ const char *months[] =
 { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug",
     "Sep", "Oct", "Nov", "Dec" };
 
-/* provided by Linus Torvalds 16-Feb-93 */
-void 
+/* Nice and simple code provided by Linus Torvalds 16-Feb-93 */
+/* Nonblocking stuff by Maciej W. Rozycki, macro@ds2.pg.gda.pl, 1999.
+   He writes: "Login performs open() on a tty in a blocking mode.
+   In some cases it may make login wait in open() for carrier infinitely,
+   for example if the line is a simplistic case of a three-wire serial
+   connection. I believe login should open the line in the non-blocking mode
+   leaving the decision to make a connection to getty (where it actually
+   belongs). */
+static void
 opentty(const char * tty)
 {
     int i;
-    int fd = open(tty, O_RDWR);
+    int fd = open(tty, O_RDWR | O_NONBLOCK);
+    int flags = fcntl(fd, F_GETFL);
+
+    flags &= ~O_NONBLOCK;
+    fcntl(fd, F_SETFL, flags);
     
     for (i = 0 ; i < fd ; i++)
       close(i);
@@ -260,24 +306,18 @@ consoletty(int fd)
 
 
 int
-main(argc, argv)
-     int argc;
-     char **argv;
+main(int argc, char **argv)
 {
-    extern int errno, optind;
+    extern int optind;
     extern char *optarg, **environ;
     struct group *gr;
     register int ch;
     register char *p;
-    int ask, fflag, hflag, pflag, cnt;
-    int quietlog, passwd_req, ioctlval;
-    char *domain, *salt, *ttyn, *pp;
+    int ask, fflag, hflag, pflag, cnt, errsv;
+    int quietlog, passwd_req;
+    char *domain, *ttyn;
     char tbuf[MAXPATHLEN + 2], tname[sizeof(_PATH_TTY) + 10];
-    char *ctime(), *ttyname(), *stypeof();
-    time_t time();
-    void timedout();
     char *termenv;
-    char vcsn[20], vcsan[20];
     char * childArgv[10];
     char * buff;
     int childArgc = 0;
@@ -286,18 +326,32 @@ main(argc, argv)
     pam_handle_t *pamh = NULL;
     struct pam_conv conv = { misc_conv, NULL };
     pid_t childPid;
-    int childStatus;
+#else
+    char *salt, *pp;
+#endif
+#ifdef CHOWNVCS
+    char vcsn[20], vcsan[20];
+#endif
+#ifndef __linux__
+    int ioctlval;
 #endif
 
     signal(SIGALRM, timedout);
     alarm((unsigned int)timeout);
     signal(SIGQUIT, SIG_IGN);
     signal(SIGINT, SIG_IGN);
+
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
     
     setpriority(PRIO_PROCESS, 0, 0);
 #ifdef HAVE_QUOTA
     quota(Q_SETUID, 0, 0, 0);
 #endif
+#ifdef DO_PS_FIDDLING
+    initproctitle(argc, argv);
+#endif
     
     /*
      * -p is used by getty to tell login not to destroy the environment
@@ -306,13 +360,13 @@ main(argc, argv)
      *    host to login so that it may be placed in utmp and wtmp
      */
     gethostname(tbuf, sizeof(tbuf));
-    strncpy(thishost, tbuf, sizeof(thishost)-1);
-    thishost[sizeof(thishost)-1] = 0;
+    xstrncpy(thishost, tbuf, sizeof(thishost));
     domain = index(tbuf, '.');
     
     username = tty = hostname = NULL;
     fflag = hflag = pflag = 0;
     passwd_req = 1;
+
     while ((ch = getopt(argc, argv, "fh:p")) != EOF)
       switch (ch) {
 	case 'f':
@@ -322,14 +376,14 @@ main(argc, argv)
 	case 'h':
 	  if (getuid()) {
 	      fprintf(stderr,
-		      "login: -h for super-user only.\n");
+		      _("login: -h for super-user only.\n"));
 	      exit(1);
 	  }
 	  hflag = 1;
 	  if (domain && (p = index(optarg, '.')) &&
 	      strcasecmp(p, domain) == 0)
 	    *p = 0;
-	  hostname = optarg;
+	  hostname = strdup(optarg); 	/* strdup: Ambrose C. Li */
 	  { 
 	      struct hostent *he = gethostbyname(hostname);
 	      if (he) {
@@ -347,17 +401,22 @@ main(argc, argv)
 	case '?':
 	default:
 	  fprintf(stderr,
-		  "usage: login [-fp] [username]\n");
+		  _("usage: login [-fp] [username]\n"));
 	  exit(1);
       }
     argc -= optind;
     argv += optind;
     if (*argv) {
-	username = *argv;
+	char *p = *argv;
+	username = strdup(p);
 	ask = 0;
+	/* wipe name - some people mistype their password here */
+	/* (of course we are too late, but perhaps this helps a little ..) */
+	while(*p)
+	    *p++ = ' ';
     } else
       ask = 1;
-    
+
 #ifndef __linux__
     ioctlval = 0;
     ioctl(0, TIOCLSET, &ioctlval);
@@ -377,7 +436,7 @@ main(argc, argv)
      */
     ioctlval = 0;
     ioctl(0, FIOSNBIO, &ioctlval);
-#endif
+#endif /* ! __linux__ */
     
     for (cnt = getdtablesize(); cnt > 2; cnt--)
       close(cnt);
@@ -389,6 +448,7 @@ main(argc, argv)
 	ttyn = tname;
     }
 
+#ifdef CHOWNVCS
     /* find names of Virtual Console devices, for later mode change */
     {
 	char *p = ttyn;
@@ -398,6 +458,7 @@ main(argc, argv)
 	strcpy(vcsn, "/dev/vcs"); strcat(vcsn, p);
 	strcpy(vcsan, "/dev/vcsa"); strcat(vcsan, p);
     }
+#endif
 
     setpgrp();
     
@@ -407,8 +468,8 @@ main(argc, argv)
 	tcgetattr(0, &tt);
 	ttt = tt;
 	ttt.c_cflag &= ~HUPCL;
-	
-	if((chown(ttyn, 0, 0) == 0) && (chmod(ttyn, 0622) == 0)) {
+
+	if((chown(ttyn, 0, 0) == 0) && (chmod(ttyn, TTY_MODE) == 0)) {
 	    tcsetattr(0,TCSAFLUSH,&ttt);
 	    signal(SIGHUP, SIG_IGN); /* so vhangup() wont kill us */
 	    vhangup();
@@ -422,25 +483,32 @@ main(argc, argv)
 	opentty(ttyn);
 	tcsetattr(0,TCSAFLUSH,&tt);
     }
-    
-    if ((tty = rindex(ttyn, '/')))
-      ++tty;
+
+    if (strncmp(ttyn, "/dev/", 5) == 0)
+	tty = ttyn+5;
     else
-      tty = ttyn;
-    
+	tty = ttyn;
+
     openlog("login", LOG_ODELAY, LOG_AUTHPRIV);
 
+#if 0
+    /* other than iso-8859-1 */
+    printf("\033(K");
+    fprintf(stderr,"\033(K");
+#endif
+
 #ifdef USE_PAM
-    /* username is initialized to NULL
-       and if specified on the command line it is set.
-       Therefore, we are safe not setting it to anything
-    */
+    /*
+     * username is initialized to NULL
+     * and if specified on the command line it is set.
+     * Therefore, we are safe not setting it to anything
+     */
 
     retcode = pam_start("login",username, &conv, &pamh);
     if(retcode != PAM_SUCCESS) {
-	fprintf(stderr,"login: PAM Failure, aborting: %s\n",
+	fprintf(stderr,_("login: PAM Failure, aborting: %s\n"),
 	pam_strerror(pamh, retcode));
-	syslog(LOG_ERR,"Couldn't initialize PAM: %s", pam_strerror(pamh, retcode));
+	syslog(LOG_ERR,_("Couldn't initialize PAM: %s"), pam_strerror(pamh, retcode));
 	exit(99);
     }
     /* hostname & tty are either set to NULL or their correct values,
@@ -449,6 +517,25 @@ main(argc, argv)
     PAM_FAIL_CHECK;
     retcode = pam_set_item(pamh, PAM_TTY, tty);
     PAM_FAIL_CHECK;
+
+    /*
+     * Andrew.Taylor@cal.montage.ca: Provide a user prompt to PAM
+     * so that the "login: " prompt gets localized. Unfortunately,
+     * PAM doesn't have an interface to specify the "Password: " string
+     * (yet).
+     */
+    retcode = pam_set_item(pamh, PAM_USER_PROMPT, _("login: "));
+    PAM_FAIL_CHECK;
+
+#if 0
+    /*
+     * other than iso-8859-1
+     * one more time due to reset tty by PAM
+     */
+    printf("\033(K");
+    fprintf(stderr,"\033(K");
+#endif
+	    
     /* if fflag == 1, then the user has already been authenticated */
     if (fflag && (getuid() == 0))
 	passwd_req = 0;
@@ -471,9 +558,9 @@ main(argc, argv)
 	       (retcode == PAM_CRED_INSUFFICIENT) ||
 	       (retcode == PAM_AUTHINFO_UNAVAIL))) {
 	    pam_get_item(pamh, PAM_USER, (const void **) &username);
-	    syslog(LOG_NOTICE,"FAILED LOGIN %d FROM %s FOR %s, %s",
-	    failcount, hostname,username,pam_strerror(pamh, retcode));
-	    fprintf(stderr,"Login incorrect\n\n");
+	    syslog(LOG_NOTICE,_("FAILED LOGIN %d FROM %s FOR %s, %s"),
+	    failcount, hostname, username, pam_strerror(pamh, retcode));
+	    fprintf(stderr,_("Login incorrect\n\n"));
 	    pam_set_item(pamh,PAM_USER,NULL);
 	    retcode = pam_authenticate(pamh, 0);
 	}
@@ -482,14 +569,14 @@ main(argc, argv)
 	    pam_get_item(pamh, PAM_USER, (const void **) &username);
 
 	    if (retcode == PAM_MAXTRIES) 
-		syslog(LOG_NOTICE,"TOO MANY LOGIN TRIES (%d) FROM %s FOR "
-			"%s, %s", failcount, hostname, username,
+		syslog(LOG_NOTICE,_("TOO MANY LOGIN TRIES (%d) FROM %s FOR "
+			"%s, %s"), failcount, hostname, username,
 			 pam_strerror(pamh, retcode));
 	    else
-		syslog(LOG_NOTICE,"FAILED LOGIN SESSION FROM %s FOR %s, %s",
+		syslog(LOG_NOTICE,_("FAILED LOGIN SESSION FROM %s FOR %s, %s"),
 			hostname, username, pam_strerror(pamh, retcode));
 
-	    fprintf(stderr,"\nLogin incorrect\n");
+	    fprintf(stderr,_("\nLogin incorrect\n"));
 	    pam_end(pamh, retcode);
 	    exit(0);
 	}
@@ -507,29 +594,36 @@ main(argc, argv)
        First get the username that we are actually using, though.
     */
     retcode = pam_get_item(pamh, PAM_USER, (const void **) &username);
-    setpwent();
-    pwd = getpwnam(username);
+    if (retcode == PAM_SUCCESS && username && *username) {
+      pwd = getpwnam(username);
+    }
+
+    /*
+     * Initialize the supplementary group list.
+     * This should be done before pam_setcred because
+     * the PAM modules might add groups during pam_setcred.
+     */
     if (pwd) initgroups(username, pwd->pw_gid);
 
-    retcode = pam_setcred(pamh, PAM_ESTABLISH_CRED);
+    retcode = pam_open_session(pamh, 0);
     PAM_FAIL_CHECK;
 
-    retcode = pam_open_session(pamh, 0);
+    retcode = pam_setcred(pamh, PAM_ESTABLISH_CRED);
     PAM_FAIL_CHECK;
 
 #else /* ! USE_PAM */
 
     for (cnt = 0;; ask = 1) {
-	ioctlval = 0;
 #  ifndef __linux__
+	ioctlval = 0;
 	ioctl(0, TIOCSETD, &ioctlval);
 #  endif
-	
+
 	if (ask) {
 	    fflag = 0;
 	    getloginname();
 	}
-	
+
 	/* Dirty patch to fix a gigantic security hole when using 
 	   yellow pages. This problem should be solved by the
 	   libraries, and not by programs, but this must be fixed
@@ -538,7 +632,7 @@ main(argc, argv)
 	   Feb 95 <alvaro@etsit.upm.es> */
 	
 	if (username[0] == '+') {
-	    puts("Illegal username");
+	    puts(_("Illegal username"));
 	    badlogin(username);
 	    sleepexit(1);
 	}
@@ -581,20 +675,20 @@ main(argc, argv)
 	 */
 	if (pwd && pwd->pw_uid == 0 && !rootterm(tty)) {
 	    fprintf(stderr,
-		    "%s login refused on this terminal.\n",
+		    _("%s login refused on this terminal.\n"),
 		    pwd->pw_name);
 	    
 	    if (hostname)
 	      syslog(LOG_NOTICE,
-		     "LOGIN %s REFUSED FROM %s ON TTY %s",
+		     _("LOGIN %s REFUSED FROM %s ON TTY %s"),
 		     pwd->pw_name, hostname, tty);
 	    else
 	      syslog(LOG_NOTICE,
-		     "LOGIN %s REFUSED ON TTY %s",
+		     _("LOGIN %s REFUSED ON TTY %s"),
 		     pwd->pw_name, tty);
 	    continue;
 	}
-	
+
 	/*
 	 * If no pre-authentication and a password exists
 	 * for this user, prompt for one and verify it.
@@ -603,7 +697,7 @@ main(argc, argv)
 	  break;
 	
 	setpriority(PRIO_PROCESS, 0, -4);
-	pp = getpass("Password: ");
+	pp = getpass(_("Password: "));
 	
 #  ifdef CRYPTOCARD
 	if (strncmp(pp, "CRYPTO", 6) == 0) {
@@ -639,10 +733,11 @@ main(argc, argv)
 	}
 #  endif /* KERBEROS */
 	memset(pp, 0, strlen(pp));
+
 	if (pwd && !strcmp(p, pwd->pw_passwd))
 	  break;
 	
-	printf("Login incorrect\n");
+	printf(_("Login incorrect\n"));
 	badlogin(username); /* log ALL bad logins */
 	failures++;
 	
@@ -664,11 +759,11 @@ main(argc, argv)
 	switch(errno) {
 	  case EUSERS:
 	    fprintf(stderr,
-		    "Too many users logged on already.\nTry again later.\n");
+		    _("Too many users logged on already.\nTry again later.\n"));
 	    break;
 	  case EPROCLIM:
 	    fprintf(stderr,
-		    "You have too many processes running.\n");
+		    _("You have too many processes running.\n"));
 	    break;
 	  default:
 	    perror("quota (Q_SETUID)");
@@ -711,78 +806,37 @@ main(argc, argv)
 	}
     }
     
-#ifndef __linux__
-#  ifdef KERBEROS
-    if (notickets && !quietlog)
-      printf("Warning: no Kerberos tickets issued\n");
-#  endif
-    
-#  ifndef USE_PAM			/* PAM does all of this for us */
-#    define TWOWEEKS	(14*24*60*60)
-    if (pwd->pw_change || pwd->pw_expire) {
-	struct timeval tp;
-
-	gettimeofday(&tp, (struct timezone *)NULL);
-
-	if (pwd->pw_change) {
-	    if (tp.tv_sec >= pwd->pw_change) {
-		printf("Sorry -- your password has expired.\n");
-		sleepexit(1);
-	    }
-	    else if (tp.tv_sec - pwd->pw_change < TWOWEEKS && !quietlog) {
-		struct tm *ttp;
-		ttp = localtime(&pwd->pw_change);
-		printf("Warning: your password expires on %s %d, %d\n",
-		       months[ttp->tm_mon], ttp->tm_mday,
-		       TM_YEAR_BASE + ttp->tm_year);
-	    }
-	}
-
-	if (pwd->pw_expire) {
-	    if (tp.tv_sec >= pwd->pw_expire) {
-		printf("Sorry -- your account has expired.\n");
-		sleepexit(1);
-	    }
-	    else if (tp.tv_sec - pwd->pw_expire < TWOWEEKS && !quietlog) {
-		struct tm *ttp;
-		ttp = localtime(&pwd->pw_expire);
-		printf("Warning: your account expires on %s %d, %d\n",
-		       months[ttp->tm_mon], ttp->tm_mday,
-		       TM_YEAR_BASE + ttp->tm_year);
-	    }
-	}
-    }
-#  endif /* !USE_PAM */
-    
-    /* nothing else left to fail -- really log in */
-    {
-	struct utmp utmp;
-	
-	memset((char *)&utmp, 0, sizeof(utmp));
-	time(&utmp.ut_time);
-	strncpy(utmp.ut_name, username, sizeof(utmp.ut_name));
-	/* ut_name may legally be non-null-terminated */
-	if (hostname) {
-	    strncpy(utmp.ut_host, hostname, sizeof(utmp.ut_host));
-	    utmp.ut_host[sizeof(utmp.ut_host)-1] = 0;
-	}
-	strncpy(utmp.ut_line, tty, sizeof(utmp.ut_line));
-	utmp.ut_line[sizeof(utmp.ut_line)-1] = 0;
-	login(&utmp);
-    }
-#else
     /* for linux, write entries in utmp and wtmp */
     {
 	struct utmp ut;
-	int wtmp;
 	struct utmp *utp;
-	time_t t;
 	pid_t mypid = getpid();
 	
 	utmpname(_PATH_UTMP);
 	setutent();
-	while ((utp = getutent()) 
-	       && !(utp->ut_pid == mypid)) /* nothing */;
+
+	/* Find mypid in utmp.
+login sometimes overwrites the runlevel entry in /var/run/utmp,
+confusing sysvinit. I added a test for the entry type, and the problem
+was gone. (In a runlevel entry, st_pid is not really a pid but some number
+calculated from the previous and current runlevel).
+Michael Riepe <michael@stud.uni-hannover.de>
+	*/
+	while ((utp = getutent()))
+		if (utp->ut_pid == mypid
+		    && utp->ut_type >= INIT_PROCESS
+		    && utp->ut_type <= DEAD_PROCESS)
+			break;
+
+	/* If we can't find a pre-existing entry by pid, try by line.
+	   BSD network daemons may rely on this. (anonymous) */
+	if (utp == NULL) {
+	     setutent();
+	     ut.ut_type = LOGIN_PROCESS;
+	     strncpy(ut.ut_id, ttyn + 8, sizeof(ut.ut_id));
+	     strncpy(ut.ut_line, ttyn + 5, sizeof(ut.ut_line));
+	     utp = getutid(&ut);
+	}
 	
 	if (utp) {
 	    memcpy(&ut, utp, sizeof(ut));
@@ -790,22 +844,26 @@ main(argc, argv)
 	    /* some gettys/telnetds don't initialize utmp... */
 	    memset(&ut, 0, sizeof(ut));
 	}
-	/* endutent(); superfluous, error for glibc */
 	
 	if (ut.ut_id[0] == 0)
 	  strncpy(ut.ut_id, ttyn + 8, sizeof(ut.ut_id));
 	
 	strncpy(ut.ut_user, username, sizeof(ut.ut_user));
-	strncpy(ut.ut_line, ttyn + 5, sizeof(ut.ut_line));
-	ut.ut_line[sizeof(ut.ut_line)-1] = 0;
-	time(&t);
-	ut.ut_time = t;		/* ut_time is not always a time_t */
-				/* (we might test #ifdef _HAVE_UT_TV ) */
+	xstrncpy(ut.ut_line, ttyn + 5, sizeof(ut.ut_line));
+#ifdef _HAVE_UT_TV		/* in <utmpbits.h> included by <utmp.h> */
+	gettimeofday(&ut.ut_tv, NULL);
+#else
+	{
+	  time_t t;
+	  time(&t);
+	  ut.ut_time = t;	/* ut_time is not always a time_t */
+				/* glibc2 #defines it as ut_tv.tv_sec */
+	}
+#endif
 	ut.ut_type = USER_PROCESS;
 	ut.ut_pid = mypid;
 	if (hostname) {
-	    strncpy(ut.ut_host, hostname, sizeof(ut.ut_host));
-	    ut.ut_host[sizeof(ut.ut_host)-1] = 0;
+	    xstrncpy(ut.ut_host, hostname, sizeof(ut.ut_host));
 	    if (hostaddress.h_addr_list)
 	      memcpy(&ut.ut_addr, hostaddress.h_addr_list[0],
 		     sizeof(ut.ut_addr));
@@ -813,9 +871,26 @@ main(argc, argv)
 	
 	pututline(&ut);
 	endutent();
-	
+
+#ifdef HAVE_updwtmp
+	updwtmp(_PATH_WTMP, &ut);
+#else
+#if 0
+	/* The O_APPEND open() flag should be enough to guarantee
+	   atomic writes at end of file. */
+	{
+	    int wtmp;
+
+	    if((wtmp = open(_PATH_WTMP, O_APPEND|O_WRONLY)) >= 0) {
+		write(wtmp, (char *)&ut, sizeof(ut));
+		close(wtmp);
+	    }
+	}
+#else
+	/* Probably all this locking below is just nonsense,
+	   and the short version is OK as well. */
 	{ 
-	    int lf;
+	    int lf, wtmp;
 	    if ((lf = open(_PATH_WTMPLOCK, O_CREAT|O_WRONLY, 0660)) >= 0) {
 		flock(lf, LOCK_EX);
 		if ((wtmp = open(_PATH_WTMP, O_APPEND|O_WRONLY)) >= 0) {
@@ -826,22 +901,17 @@ main(argc, argv)
 		close(lf);
 	    }
 	}
-    }
 #endif
+#endif
+    }
     
     dolastlog(quietlog);
     
-#ifndef __linux__
-    if (!hflag) {					/* XXX */
-	static struct winsize win = { 0, 0, 0, 0 };
-	
-	ioctl(0, TIOCSWINSZ, &win);
-    }
-#endif
     chown(ttyn, pwd->pw_uid,
 	  (gr = getgrnam(TTYGRPNAME)) ? gr->gr_gid : pwd->pw_gid);
     chmod(ttyn, TTY_MODE);
 
+#ifdef CHOWNVCS
     /* if tty is one of the VC's then change owner and mode of the 
        special /dev/vcs devices as well */
     if (consoletty(0)) {
@@ -850,6 +920,7 @@ main(argc, argv)
 	chmod(vcsn, TTY_MODE);
 	chmod(vcsan, TTY_MODE);
     }
+#endif
 
     setgid(pwd->pw_gid);
     
@@ -882,17 +953,6 @@ main(argc, argv)
 	  memset(environ, 0, sizeof(char*));
       }
     
-#ifndef __linux__
-    setenv("HOME", pwd->pw_dir, 1);
-    setenv("SHELL", pwd->pw_shell, 1);
-    if (term[0] == '\0') {
-	strncpy(term, stypeof(tty), sizeof(term));
-	term[sizeof(term)-1] = 0;
-    }
-    setenv("TERM", term, 0);
-    setenv("USER", pwd->pw_name, 1);
-    setenv("PATH", _PATH_DEFPATH, 0);
-#else
     setenv("HOME", pwd->pw_dir, 0);      /* legal to override */
     if(pwd->pw_uid)
       setenv("PATH", _PATH_DEFPATH, 1);
@@ -903,7 +963,8 @@ main(argc, argv)
     setenv("TERM", termenv, 1);
     
     /* mailx will give a funny error msg if you forget this one */
-    { char tmp[MAXPATHLEN];
+    {
+      char tmp[MAXPATHLEN];
       /* avoid snprintf */
       if (sizeof(_PATH_MAILDIR) + strlen(pwd->pw_name) + 1 < MAXPATHLEN) {
 	      sprintf(tmp, "%s/%s", _PATH_MAILDIR, pwd->pw_name);
@@ -915,7 +976,6 @@ main(argc, argv)
        HP-UX 6.5 does it. We'll not allow modifying it.
        */
     setenv("LOGNAME", pwd->pw_name, 1);
-#endif
 
 #ifdef USE_PAM
     {
@@ -932,67 +992,97 @@ main(argc, argv)
 	}
     }
 #endif
+
+#ifdef DO_PS_FIDDLING
+    setproctitle("login", username);
+#endif
     
     if (tty[sizeof("tty")-1] == 'S')
-      syslog(LOG_INFO, "DIALUP AT %s BY %s", tty, pwd->pw_name);
+      syslog(LOG_INFO, _("DIALUP AT %s BY %s"), tty, pwd->pw_name);
     
     /* allow tracking of good logins.
        -steve philp (sphilp@mail.alliance.net) */
     
     if (pwd->pw_uid == 0) {
 	if (hostname)
-	  syslog(LOG_NOTICE, "ROOT LOGIN ON %s FROM %s",
+	  syslog(LOG_NOTICE, _("ROOT LOGIN ON %s FROM %s"),
 		 tty, hostname);
 	else
-	  syslog(LOG_NOTICE, "ROOT LOGIN ON %s", tty);
+	  syslog(LOG_NOTICE, _("ROOT LOGIN ON %s"), tty);
     } else {
 	if (hostname) 
-	  syslog(LOG_INFO, "LOGIN ON %s BY %s FROM %s", tty, 
+	  syslog(LOG_INFO, _("LOGIN ON %s BY %s FROM %s"), tty, 
 		 pwd->pw_name, hostname);
 	else 
-	  syslog(LOG_INFO, "LOGIN ON %s BY %s", tty, 
+	  syslog(LOG_INFO, _("LOGIN ON %s BY %s"), tty, 
 		 pwd->pw_name);
     }
     
     if (!quietlog) {
 	struct stat st;
+	char *mail;
 	
 	motd();
-	/* avoid snprintf */
-	if (sizeof(_PATH_MAILDIR) + strlen(pwd->pw_name) + 1 < sizeof(tbuf)) {
-		sprintf(tbuf, "%s/%s", _PATH_MAILDIR, pwd->pw_name);
-		if (stat(tbuf, &st) == 0 && st.st_size != 0)
-			printf("You have %smail.\n",
-			       (st.st_mtime > st.st_atime) ? "new " : "");
+	mail = getenv("MAIL");
+	if (mail && stat(mail, &st) == 0 && st.st_size != 0
+		 && st.st_size != REDHAT_IGNORED_MAILSIZE) {
+		printf(_("You have %smail.\n"),
+		       (st.st_mtime > st.st_atime) ? _("new ") : "");
 	}
     }
     
     signal(SIGALRM, SIG_DFL);
     signal(SIGQUIT, SIG_DFL);
-    signal(SIGINT, SIG_DFL);
     signal(SIGTSTP, SIG_IGN);
     signal(SIGHUP, SIG_DFL);
+
+#ifdef USE_PAM
+    /*
+     * We must fork before setuid() because we need to call
+     * pam_close_session() as root.
+     */
+    /*
+     * Problem: if the user's shell is a shell like ash that doesnt do
+     * setsid() or setpgrp(), then a ctrl-\, sending SIGQUIT to every
+     * process in the pgrp, will kill us.
+     * Solution: use TIOCNOTTY and setsid().
+     */
+    signal(SIGINT, SIG_IGN);
+    childPid = fork();
+    if (childPid < 0) {
+       int errsv = errno;
+       /* error in fork() */
+       fprintf(stderr,_("login: failure forking: %s"), strerror(errsv));
+       PAM_END;
+       exit(0);
+    } else if (childPid) {
+       /* parent - wait for child to finish, then cleanup session */
+       signal(SIGHUP, SIG_IGN);		/* ignore signal from TIOCNOTTY */
+       ioctl(0, TIOCNOTTY, NULL);
+       signal(SIGHUP, SIG_DFL);
+
+       wait(NULL);
+       PAM_END;
+       exit(0);
+    }
+    /* child */
+    setsid();
+#endif
+    signal(SIGINT, SIG_DFL);
     
     /* discard permissions last so can't get killed and drop core */
     if(setuid(pwd->pw_uid) < 0 && pwd->pw_uid) {
-	syslog(LOG_ALERT, "setuid() failed");
-#ifdef USE_PAM
-	PAM_END;
-#endif
+	syslog(LOG_ALERT, _("setuid() failed"));
 	exit(1);
     }
     
     /* wait until here to change directory! */
     if (chdir(pwd->pw_dir) < 0) {
-	printf("No directory %s!\n", pwd->pw_dir);
-	if (chdir("/")) {
-#ifdef USE_PAM
-	  PAM_END;
-#endif
+	printf(_("No directory %s!\n"), pwd->pw_dir);
+	if (chdir("/"))
 	  exit(0);
-	}
 	pwd->pw_dir = "/";
-	printf("Logging in with home = \"/\".\n");
+	printf(_("Logging in with home = \"/\".\n"));
     }
     
     /* if the shell field has a space: treat it like a shell script */
@@ -1000,7 +1090,7 @@ main(argc, argv)
 	buff = malloc(strlen(pwd->pw_shell) + 6);
 
 	if (!buff) {
-	    fprintf(stderr, "login: no memory for shell script.\n");
+	    fprintf(stderr, _("login: no memory for shell script.\n"));
 	    exit(0);
 	}
 
@@ -1012,10 +1102,9 @@ main(argc, argv)
 	childArgv[childArgc++] = buff;
     } else {
 	tbuf[0] = '-';
-	strncpy(tbuf + 1, ((p = rindex(pwd->pw_shell, '/')) ?
+	xstrncpy(tbuf + 1, ((p = rindex(pwd->pw_shell, '/')) ?
 			   p + 1 : pwd->pw_shell),
 		sizeof(tbuf)-1);
-	tbuf[sizeof(tbuf)-1] = 0;
 	
 	childArgv[childArgc++] = pwd->pw_shell;
 	childArgv[childArgc++] = tbuf;
@@ -1023,38 +1112,30 @@ main(argc, argv)
 
     childArgv[childArgc++] = NULL;
 
-#ifdef USE_PAM 
-    /* There was some junk with fork()/exec()/signal()/wait() here
-       that was incorrect, and util-linux-2.7-11.src.rpm contains
-       a patch that makes the fork entirely useless.
-       If you introduce one again, please document in the source
-       what its purpose is. - aeb */
-    PAM_END;
-#endif /* USE_PAM */
-
     execvp(childArgv[0], childArgv + 1);
 
-    if (!strcmp(childArgv[0], "/bin/sh")) 
-	    fprintf(stderr, "login: couldn't exec shell script: %s.\n",
-		    strerror(errno));
+    errsv = errno;
+
+    if (!strcmp(childArgv[0], "/bin/sh"))
+	fprintf(stderr, _("login: couldn't exec shell script: %s.\n"),
+		strerror(errsv));
     else
-	    fprintf(stderr, "login: no shell: %s.\n", strerror(errno));
+	fprintf(stderr, _("login: no shell: %s.\n"), strerror(errsv));
 
     exit(0);
 }
 
-void
-getloginname()
-{
-    register int ch;
-    register char *p;
+#ifndef USE_PAM
+static void
+getloginname(void) {
+    int ch, cnt, cnt2;
+    char *p;
     static char nbuf[UT_NAMESIZE + 1];
-    int cnt, cnt2;
     
     cnt2 = 0;
     for (;;) {
 	cnt = 0;
-	printf("\n%s login: ", thishost); fflush(stdout);
+	printf(_("\n%s login: "), thishost); fflush(stdout);
 	for (p = nbuf; (ch = getchar()) != '\n'; ) {
 	    if (ch == EOF) {
 		badlogin("EOF");
@@ -1065,36 +1146,37 @@ getloginname()
 	    
 	    cnt++;
 	    if (cnt > UT_NAMESIZE + 20) {
-		fprintf(stderr, "login name much too long.\n");
-		badlogin("NAME too long");
+		fprintf(stderr, _("login name much too long.\n"));
+		badlogin(_("NAME too long"));
 		exit(0);
 	    }
 	}
-	if (p > nbuf)
+	if (p > nbuf) {
 	  if (nbuf[0] == '-')
 	    fprintf(stderr,
-		    "login names may not start with '-'.\n");
+		    _("login names may not start with '-'.\n"));
 	  else {
 	      *p = '\0';
 	      username = nbuf;
 	      break;
 	  }
+	}
 	
 	cnt2++;
 	if (cnt2 > 50) {
-	    fprintf(stderr, "too many bare linefeeds.\n");
-	    badlogin("EXCESSIVE linefeeds");
+	    fprintf(stderr, _("too many bare linefeeds.\n"));
+	    badlogin(_("EXCESSIVE linefeeds"));
 	    exit(0);
 	}
     }
 }
+#endif
 
 void
-timedout()
-{
+timedout(int sig) {
     struct termio ti;
     
-    fprintf(stderr, "Login timed out after %d seconds\n", timeout);
+    fprintf(stderr, _("Login timed out after %d seconds\n"), timeout);
     
     /* reset echo */
     ioctl(0, TCGETA, &ti);
@@ -1103,9 +1185,9 @@ timedout()
     exit(0);
 }
 
+#ifndef USE_PAM
 int
-rootterm(ttyn)
-     char *ttyn;
+rootterm(char * ttyn)
 #ifndef __linux__
 {
     struct ttyent *t;
@@ -1116,7 +1198,7 @@ rootterm(ttyn)
 { 
     int fd;
     char buf[100],*p;
-    int cnt, more;
+    int cnt, more = 0;
     
     fd = open(SECURETTY, O_RDONLY);
     if(fd < 0) return 1;
@@ -1140,15 +1222,15 @@ rootterm(ttyn)
   	}
     }
 }
-#endif
+#endif /* !__linux__ */
+#endif /* !USE_PAM */
 
 jmp_buf motdinterrupt;
 
 void
-motd()
-{
-    register int fd, nchars;
-    void (*oldint)(), sigint();
+motd(void) {
+    int fd, nchars;
+    void (*oldint)(int);
     char tbuf[8192];
     
     if ((fd = open(_PATH_MOTDFILE, O_RDONLY, 0)) < 0)
@@ -1162,15 +1244,13 @@ motd()
 }
 
 void
-sigint()
-{
+sigint(int sig) {
     longjmp(motdinterrupt, 1);
 }
 
 #ifndef USE_PAM				/* PAM takes care of this */
 void
-checknologin()
-{
+checknologin(void) {
     register int fd, nchars;
     char tbuf[8192];
     
@@ -1183,37 +1263,33 @@ checknologin()
 #endif
 
 void
-dolastlog(quiet)
-     int quiet;
-{
+dolastlog(int quiet) {
     struct lastlog ll;
     int fd;
     
     if ((fd = open(_PATH_LASTLOG, O_RDWR, 0)) >= 0) {
-	lseek(fd, (off_t)pwd->pw_uid * sizeof(ll), L_SET);
+	lseek(fd, (off_t)pwd->pw_uid * sizeof(ll), SEEK_SET);
 	if (!quiet) {
 	    if (read(fd, (char *)&ll, sizeof(ll)) == sizeof(ll) &&
 		ll.ll_time != 0) {
-		printf("Last login: %.*s ",
+		printf(_("Last login: %.*s "),
 		       24-5, (char *)ctime(&ll.ll_time));
 		
 		if (*ll.ll_host != '\0')
-		  printf("from %.*s\n",
+		  printf(_("from %.*s\n"),
 			 (int)sizeof(ll.ll_host), ll.ll_host);
 		else
-		  printf("on %.*s\n",
+		  printf(_("on %.*s\n"),
 			 (int)sizeof(ll.ll_line), ll.ll_line);
 	    }
-	    lseek(fd, (off_t)pwd->pw_uid * sizeof(ll), L_SET);
+	    lseek(fd, (off_t)pwd->pw_uid * sizeof(ll), SEEK_SET);
 	}
 	memset((char *)&ll, 0, sizeof(ll));
 	time(&ll.ll_time);
-	strncpy(ll.ll_line, tty, sizeof(ll.ll_line));
-	ll.ll_line[sizeof(ll.ll_line)-1] = 0;
-	if (hostname) {
-	    strncpy(ll.ll_host, hostname, sizeof(ll.ll_host));
-	    ll.ll_host[sizeof(ll.ll_host)-1] = 0;
-	}
+	xstrncpy(ll.ll_line, tty, sizeof(ll.ll_line));
+	if (hostname)
+	    xstrncpy(ll.ll_host, hostname, sizeof(ll.ll_host));
+
 	write(fd, (char *)&ll, sizeof(ll));
 	close(fd);
     }
@@ -1222,12 +1298,21 @@ dolastlog(quiet)
 void
 badlogin(const char *name)
 {
-    if (hostname)
-      syslog(LOG_NOTICE, "%d LOGIN FAILURE%s FROM %s, %s",
-	     failures, failures > 1 ? "S" : "", hostname, name);
-    else
-      syslog(LOG_NOTICE, "%d LOGIN FAILURE%s ON %s, %s",
-	     failures, failures > 1 ? "S" : "", tty, name);
+    if (failures == 1) {
+	if (hostname)
+	  syslog(LOG_NOTICE, _("LOGIN FAILURE FROM %s, %s"),
+		 hostname, name);
+	else
+	  syslog(LOG_NOTICE, _("LOGIN FAILURE ON %s, %s"),
+		 tty, name);
+    } else {
+	if (hostname)
+	  syslog(LOG_NOTICE, _("%d LOGIN FAILURES FROM %s, %s"),
+		 failures, hostname, name);
+	else
+	  syslog(LOG_NOTICE, _("%d LOGIN FAILURES ON %s, %s"),
+		 failures, tty, name);
+    }
 }
 
 #undef	UNKNOWN
@@ -1235,9 +1320,7 @@ badlogin(const char *name)
 
 #ifndef __linux__
 char *
-stypeof(ttyid)
-     char *ttyid;
-{
+stypeof(char *ttyid) {
     struct ttyent *t;
 
     return(ttyid && (t = getttynam(ttyid)) ? t->ty_type : UNKNOWN);
@@ -1246,9 +1329,7 @@ stypeof(ttyid)
 
 /* should not be called from PAM code... Why? */
 void
-sleepexit(eval)
-     int eval;
-{
+sleepexit(int eval) {
     sleep(SLEEP_EXIT_TIMEOUT);
     exit(eval);
 }
