@@ -3,17 +3,12 @@
  *
  * Initially written for Linux by Peter Orbaek <poe@daimi.aau.dk>
  * Currently maintained at ftp://ftp.daimi.aau.dk/pub/linux/poe/
-
- Hacked by Alvaro Martinez Echevarria, alvaro@enano.etsit.upm.es,
- to allow peaceful coexistence with yp. Nov 94.
-
- Hacked to allow root to set passwd from command line.
- by Arpad Magossanyi (mag@tas.vein.hu) 
-
- Hacked by Peter Breitenlohner, peb@mppmu.mpg.de,
- moved Alvaro's changes to setpwnam.c (so they get used
- by chsh and chfn as well). Oct 5, 96.
-
+ *
+ * Hacked by Alvaro Martinez Echevarria, alvaro@enano.etsit.upm.es,
+ * to allow peaceful coexistence with yp. Nov 94.
+ *
+ * Hacked to allow root to set passwd from command line.
+ * by Arpad Magossanyi (mag@tas.vein.hu)
  */
 
 /*
@@ -26,7 +21,7 @@
 	or running a background job that just collects all command lines)
 
  *	work and second I wanted simplicity checks to be done for
- *	root, too. Only root can turn this of using the -f
+ *	root, too. Only root can turn this off using the -f
  *	switch. Okay, I started with this to support -V version
  *	information, but one thing comes to the next. *sigh*
  *	In a later step perhaps we'll be able to support shadow
@@ -43,6 +38,12 @@
  *	auth.warning. (Of course, the password itself is not logged.)
  */
 
+ /* 1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+  * - added Native Language Support
+  * Sun Mar 21 1999 - Arnaldo Carvalho de Melo <acme@conectiva.com.br>
+  * - fixed strerr(errno) in gettext calls
+  */
+
 /*
  * Usage: passwd [username [password]]
  * Only root may use the one and two argument forms. 
@@ -63,13 +64,13 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/resource.h>
+#include <stdlib.h>
 #include "my_crypt.h"
-
-#if 0
-#  include "../version.h"
-#else
-char version[] = "admutil 1.18, 15-Oct-95";
-#endif
+#include "setpwnam.h"
+#include "islocal.h"
+#include "xstrncpy.h"
+#include "nls.h"
+#include "env.h"
 
 #ifndef _PATH_CHFN
 # define _PATH_CHFN "/usr/bin/chfn"
@@ -82,9 +83,6 @@ char version[] = "admutil 1.18, 15-Oct-95";
 #include <syslog.h>
 #endif /* LOGALL */
 
-extern int is_local(char *);		/* islocal.c */
-extern int setpwnam(struct passwd *);	/* setpwnam.c */
-
 #define ascii_to_bin(c) ((c)>='a'?(c-59):(c)>='A'?((c)-53):(c)-'.')
 #define bin_to_ascii(c) ((c)>=38?((c)-38+'a'):(c)>=12?((c)-12+'A'):(c)+'.')
 
@@ -107,9 +105,8 @@ pexit(char *str, ...)
  * This would probably be the best place for checking against 
  * dictionaries. :-)
  */
-
-int check_passwd_string(char *passwd, char *string)
-{
+static int
+check_passwd_string(char *passwd, char *string) {
     int r;
     char *p, *q;
 
@@ -155,13 +152,13 @@ int check_passwd_string(char *passwd, char *string)
     return 1;
 }
 	
-int check_passwd(char *passwd, char *oldpasswd, char *user, char *gecos)
-{
+static int
+check_passwd(char *passwd, char *oldpasswd, char *user, char *gecos) {
     int ucase, lcase, digit, other;
     char *c, *g, *p;
 
     if ( (strlen(passwd) < 6) ) {
-	printf("The password must have at least 6 characters, try again.\n");
+	printf(_("The password must have at least 6 characters, try again.\n"));
 	return 0;
     }
 	
@@ -174,19 +171,19 @@ int check_passwd(char *passwd, char *oldpasswd, char *user, char *gecos)
     }
 	
     if ( (other + digit + ucase + lcase) < 2) {
-	printf("The password must contain characters out of two of the following\n");
-	printf("classes:  upper and lower case letters, digits and non alphanumeric\n");
-	printf("characters. See passwd(1) for more information.\n");
+	printf(_("The password must contain characters out of two of the following\n"));
+	printf(_("classes:  upper and lower case letters, digits and non alphanumeric\n"));
+	printf(_("characters. See passwd(1) for more information.\n"));
 	return 0;
     }
 	
     if ( oldpasswd[0] && !strncmp(oldpasswd, crypt(passwd, oldpasswd), 13) ) {
-	printf("You cannot reuse the old password.\n");
+	printf(_("You cannot reuse the old password.\n"));
 	return 0;
     }
 	
     if ( !check_passwd_string(passwd, user) ) {
-	printf("Please don't use something like your username as password!\n");
+	printf(_("Please don't use something like your username as password!\n"));
 	return 0;
     }
 
@@ -197,14 +194,14 @@ int check_passwd(char *passwd, char *oldpasswd, char *user, char *gecos)
 	    g[c-gecos] = 0;
 	    while ( (c=rindex(g, ' ')) ) {
 		if ( !check_passwd_string(passwd, c+1) ) {
-		    printf("Please don't use something like your realname as password!\n");
+		    printf(_("Please don't use something like your realname as password!\n"));
 		    free (g);
 		    return 0;
 		}
 		*c = '\0';
 	    } /* while */
 	    if ( !check_passwd_string(passwd, g) ) {
-		printf("Please don't use something like your realname as password!\n");
+		printf(_("Please don't use something like your realname as password!\n"));
 		free (g);
 		return 0;
 	    }
@@ -219,17 +216,16 @@ int check_passwd(char *passwd, char *oldpasswd, char *user, char *gecos)
     return 1; /* fine */
 }
 
-void usage()
-{
-    printf ("Usage: passwd [username [password]]\n");
-    printf("Only root may use the one and two argument forms.\n");
+#if 0
+static void
+usage(void) {
+    printf (_("Usage: passwd [username [password]]\n"));
+    printf(_("Only root may use the one and two argument forms.\n"));
 }
+#endif
 
 int
-main(argc, argv)
-     int argc;
-     char *argv[];
-{
+main(int argc, char *argv[]) {
     struct passwd *pe;
     uid_t gotuid = getuid();
     char *pwdstr = NULL, *cryptstr, *oldstr;
@@ -253,6 +249,11 @@ main(argc, argv)
 	{0, 0, 0, 0}
 	};
 
+    sanitize_env();
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
+
     optind = 0;
     while ( (c = getopt_long(argc, argv, "foqsvV", long_options, &opt_index)) != -1 ) {
 	switch (c) {
@@ -270,17 +271,17 @@ main(argc, argv)
 	    break;
 	case 'V':
 	case 'v':
-	    printf("%s\n", version);
+	    printf("%s\n", util_linux_version);
 	    exit(0);
 	default:
-	    fprintf(stderr, "Usage: passwd [-foqsvV] [user [password]]\n");
+	    fprintf(stderr, _("Usage: passwd [-foqsvV] [user [password]]\n"));
 	    exit(1);
 	} /* switch (c) */
     } /* while */
 
     if (fullname || shell) {
 	char *args[100];
-	int i, j;
+	int i, j, errsv;
 
 	setuid(getuid()); /* drop special privs. */
 	if (fullname)
@@ -293,29 +294,33 @@ main(argc, argv)
 
 	args[j] = NULL;
 	execv(args[0], args);
-	fprintf(stderr, "Can't exec %s: %s\n", args[0], strerror(errno));
+	errsv = errno;
+	fprintf(stderr, _("Can't exec %s: %s\n"), args[0], strerror(errsv));
 	exit(1);
     }
     
     switch (argc - optind) {
     case 0:
-	if ( !(user = getlogin()) ) {
+	/* Why use getlogin()? Some systems allow having several
+	   usernames with the same uid, especially several root accounts.
+	   One changes the password for the username, not the uid. */
+	if ( !(user = getlogin()) || !*user ) {
 	    if ( !(pe = getpwuid( getuid() )) ) {
-		pexit("Cannot find login name");
+		pexit(_("Cannot find login name"));
 	    } else
 		user = pe->pw_name;
 	}
 	break;
     case 1:
 	if(gotuid) {
-	    printf("Only root can change the password for others.\n");
+	    printf(_("Only root can change the password for others.\n"));
 	    exit (1);
 	} else
 	    user = argv[optind];
 	break;
     case 2:
 	if(gotuid) {
-	    printf("Only root can change the password for others.\n");
+	    printf(_("Only root can change the password for others.\n"));
 	    exit(1);
 	} else {
 	    user = argv[optind];
@@ -323,33 +328,33 @@ main(argc, argv)
 	}
 	break;
     default:
-	printf("Too many arguments.\n");
+	printf(_("Too many arguments.\n"));
 	exit (1);
     } /* switch */
 
     if(!(pe = getpwnam(user))) {
-	pexit("Can't find username anywhere. Is `%s' really a user?", user);
+	pexit(_("Can't find username anywhere. Is `%s' really a user?"), user);
     }
     
     if (!(is_local(user))) {
-	puts("Sorry, I can only change local passwords. Use yppasswd instead.");
+	puts(_("Sorry, I can only change local passwords. Use yppasswd instead."));
 	exit(1);
     }
     
     /* if somebody got into changing utmp... */
     if(gotuid && gotuid != pe->pw_uid) {
-	puts("UID and username does not match, imposter!");
+	puts(_("UID and username does not match, imposter!"));
 	exit(1);
     }
     
     if ( !silent )
-	printf( "Changing password for %s\n", user );
+	printf( _("Changing password for %s\n"), user );
     
     if ( (gotuid && pe->pw_passwd && pe->pw_passwd[0]) 
 	|| (!gotuid && !strcmp(user,"root")) ) {
-	oldstr = getpass("Enter old password: ");
+	oldstr = getpass(_("Enter old password: "));
 	if(strncmp(pe->pw_passwd, crypt(oldstr, pe->pw_passwd), 13)) {
-	    puts("Illegal password, imposter.");
+	    puts(_("Illegal password, imposter."));
 	    exit(1);
 	}
     }
@@ -361,9 +366,9 @@ main(argc, argv)
 	/* password not set on command line by root, ask for it ... */
 	
       redo_it:
-	pwdstr = getpass("Enter new password: ");
+	pwdstr = getpass(_("Enter new password: "));
 	if (pwdstr[0] == '\0') {
-	    puts("Password not changed.");
+	    puts(_("Password not changed."));
 	    exit(1);
 	}
 
@@ -371,12 +376,11 @@ main(argc, argv)
 	     && !check_passwd(pwdstr, pe->pw_passwd, user, pe->pw_gecos) ) 
 	    goto redo_it;
 	
-	strncpy(pwdstr1, pwdstr, 9);
-	pwdstr1[9] = 0;
-	pwdstr = getpass("Re-type new password: ");
+	xstrncpy(pwdstr1, pwdstr, sizeof(pwdstr1));
+	pwdstr = getpass(_("Re-type new password: "));
 	
 	if(strncmp(pwdstr, pwdstr1, 8)) {
-	    puts("You misspelled it. Password not changed.");
+	    puts(_("You misspelled it. Password not changed."));
 	    exit(1);
 	}
     } /* pwdstr i.e. password set on command line */
@@ -391,28 +395,28 @@ main(argc, argv)
 #ifdef LOGALL
     openlog("passwd", 0, LOG_AUTH);
     if (gotuid)
-	syslog(LOG_NOTICE,"password changed, user %s",user);
+	syslog(LOG_NOTICE,_("password changed, user %s"),user);
     else {
 	if ( !strcmp(user, "root") )
-	    syslog(LOG_WARNING,"ROOT PASSWORD CHANGED");
+	    syslog(LOG_WARNING,_("ROOT PASSWORD CHANGED"));
 	else
-	    syslog(LOG_NOTICE,"password changed by root, user %s",user);
+	    syslog(LOG_NOTICE,_("password changed by root, user %s"),user);
     }
     closelog();
 #endif /* LOGALL */
 
     pe->pw_passwd = cryptstr;
 #ifdef DEBUG
-    printf ("calling setpwnam to set password.\n");
+    printf (_("calling setpwnam to set password.\n"));
 #else
     if (setpwnam( pe ) < 0) {
        perror( "setpwnam" );
-       printf( "Password *NOT* changed.  Try again later.\n" );
+       printf( _("Password *NOT* changed.  Try again later.\n" ));
        exit( 1 );
     }
 #endif
 
     if ( !silent )
-	printf("Password changed.\n");	
+	printf(_("Password changed.\n"));	
     exit(0);
 }
