@@ -37,6 +37,11 @@
  * <joey@finlandia.infodrom.north.de>.  Support for vigr.
  *
  * Martin Schulze's patches adapted to Util-Linux by Nicolai Langfeldt.
+ *
+ * 1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+ * - added Native Language Support
+ * Sun Mar 21 1999 - Arnaldo Carvalho de Melo <acme@conectiva.com.br>
+ * - fixed strerr(errno) in gettext calls
  */
 
 static char version_string[] = "vipw 1.4";
@@ -59,6 +64,8 @@ static char version_string[] = "vipw 1.4";
 #include <unistd.h>
 
 #include "setpwnam.h"
+#include "xstrncpy.h"
+#include "nls.h"
 
 #define FILENAMELEN 67
 
@@ -71,11 +78,9 @@ char tmptmp_file[FILENAMELEN]; /* very tmp file */
 
 void pw_error __P((char *, int, int));
 
-void
-copyfile(from, to)
-	register int from, to;
-{
-	register int nr, nw, off;
+static void
+copyfile(int from, int to) {
+	int nr, nw, off;
 	char buf[8*1024];
 	
 	while ((nr = read(from, buf, sizeof(buf))) > 0)
@@ -88,9 +93,8 @@ copyfile(from, to)
 }
 
 
-void
-pw_init()
-{
+static void
+pw_init(void) {
 	struct rlimit rlim;
 
 	/* Unlimited resource limits. */
@@ -119,9 +123,8 @@ pw_init()
 	(void)umask(0);
 }
 
-int
-pw_lock()
-{
+static int
+pw_lock(void) {
 	int lockfd, fd, ret;
 
 	/* 
@@ -130,17 +133,10 @@ pw_lock()
 	 * that users can't get at the encrypted passwords while editing.
 	 * Open should allow flock'ing the file; see 4.4BSD.	XXX
 	 */
-	lockfd = open(orig_file, O_RDONLY, 0);
-
-	if (lockfd < 0) {
-		(void)fprintf(stderr, "%s: %s: %s\n",
-		    progname, orig_file, strerror(errno));
-		exit(1);
-	}
 #if 0 /* flock()ing is superfluous here, with the ptmp/ptmptmp system. */
 	if (flock(lockfd, LOCK_EX|LOCK_NB)) {
 		(void)fprintf(stderr,
-		    "%s: the %s file is busy.\n", progname, 
+		    _("%s: the %s file is busy.\n"), progname, 
 			      program == VIPW ? "password" : "group" );
 		exit(1);
 	}
@@ -156,49 +152,59 @@ pw_lock()
 	if (ret == -1) {
 	    if (errno == EEXIST)
 		(void)fprintf(stderr, 
-			      "%s: the %s file is busy (%s present)\n",
+			      _("%s: the %s file is busy (%s present)\n"),
 			      progname,
 			      program == VIPW ? "password" : "group",
 			      tmp_file);
-	    else
-		(void)fprintf(stderr, "%s: can't link %s: %s\n", progname,
-			      tmp_file, strerror(errno));
+	    else {
+		int errsv = errno;
+		(void)fprintf(stderr, _("%s: can't link %s: %s\n"), progname,
+			      tmp_file, strerror(errsv));
+	    }
 	    exit(1);
 	}
+
+	lockfd = open(orig_file, O_RDONLY, 0);
+
+	if (lockfd < 0) {
+		(void)fprintf(stderr, "%s: %s: %s\n",
+		    progname, orig_file, strerror(errno));
+		unlink(tmp_file);
+		exit(1);
+	}
+
 	copyfile(lockfd, fd);
 	(void)close(lockfd);
 	(void)close(fd);
 	return(1);
 }
 
-void
-pw_unlock()
-{
+static void
+pw_unlock(void) {
   char tmp[FILENAMELEN];
   
   sprintf(tmp, "%s%s", orig_file, ".OLD");
   unlink(tmp);
   link(orig_file, tmp);
   if (rename(tmp_file, orig_file) == -1) {
+    int errsv = errno;
     (void)fprintf(stderr, 
-		  "%s: can't unlock %s: %s (your changes are still in %s)\n", 
-		  progname, orig_file, strerror(errno), tmp_file);
+		  _("%s: can't unlock %s: %s (your changes are still in %s)\n"), 
+		  progname, orig_file, strerror(errsv), tmp_file);
     exit(1);
   }
   (void)unlink(tmp_file);
 }
 
 
-void
-pw_edit(notsetuid)
-	int notsetuid;
-{
+static void
+pw_edit(int notsetuid) {
 	int pstat;
 	pid_t pid;
 	char *p, *editor;
 
 	if (!(editor = getenv("EDITOR")))
-		editor = _PATH_VI;
+		editor = strdup(_PATH_VI); /* adia@egnatia.ee.auth.gr */
 	if ((p = strrchr(strtok(editor," \t"), '/')) != NULL)
 		++p;
 	else 
@@ -206,7 +212,7 @@ pw_edit(notsetuid)
 
 	pid = fork();
 	if (pid < 0) {
-		(void)fprintf(stderr, "%s: Cannot fork\n", progname);
+		(void)fprintf(stderr, _("%s: Cannot fork\n"), progname);
 		exit(1);
 	}
 	if (!pid) {
@@ -246,28 +252,50 @@ pw_error(name, err, eval)
 		(void)fprintf(stderr, "%s\n", strerror(sverrno));
 	}
 	(void)fprintf(stderr,
-	    "%s: %s unchanged\n", progname, orig_file);
+	    _("%s: %s unchanged\n"), progname, orig_file);
 	(void)unlink(tmp_file);
 	exit(eval);
 }
 
+static void
+edit_file(void)
+{
+	struct stat begin, end;
+
+	pw_init();
+	pw_lock();
+
+	if (stat(tmp_file, &begin))
+		pw_error(tmp_file, 1, 1);
+	pw_edit(0);
+	if (stat(tmp_file, &end))
+		pw_error(tmp_file, 1, 1);
+	if (begin.st_mtime == end.st_mtime) {
+		(void)fprintf(stderr, _("%s: no changes made\n"), progname);
+		pw_error((char *)NULL, 0, 0);
+	}
+	pw_unlock();
+}
+
 int main(int argc, char *argv[])
 {
-  struct stat begin, end;
+
+  setlocale(LC_ALL, "");
+  bindtextdomain(PACKAGE, LOCALEDIR);
+  textdomain(PACKAGE);
 
   bzero(tmp_file, FILENAMELEN);
   progname = (rindex(argv[0], '/')) ? rindex(argv[0], '/') + 1 : argv[0];
   if (!strcmp(progname, "vigr")) {
     program = VIGR;
-    strncpy(orig_file, GROUP_FILE, FILENAMELEN-1);
-    strncpy(tmp_file, GTMP_FILE, FILENAMELEN-1);
-    strncpy(tmptmp_file, GTMPTMP_FILE, FILENAMELEN-1);
-  }
-  else {
+    xstrncpy(orig_file, GROUP_FILE, sizeof(orig_file));
+    xstrncpy(tmp_file, GTMP_FILE, sizeof(tmp_file));
+    xstrncpy(tmptmp_file, GTMPTMP_FILE, sizeof(tmptmp_file));
+  } else {
     program = VIPW;
-    strncpy(orig_file, PASSWD_FILE, FILENAMELEN-1);
-    strncpy(tmp_file, PTMP_FILE, FILENAMELEN-1);
-    strncpy(tmptmp_file, PTMPTMP_FILE, FILENAMELEN-1);
+    xstrncpy(orig_file, PASSWD_FILE, sizeof(orig_file));
+    xstrncpy(tmp_file, PTMP_FILE, sizeof(tmp_file));
+    xstrncpy(tmptmp_file, PTMPTMP_FILE, sizeof(tmptmp_file));
   }
 
   if ((argc > 1) && 
@@ -276,18 +304,32 @@ int main(int argc, char *argv[])
     exit(0);
   }
 
-  pw_init();
-  pw_lock();
+  edit_file();
 
-  if (stat(tmp_file, &begin))
-    pw_error(tmp_file, 1, 1);
-  pw_edit(0);
-  if (stat(tmp_file, &end))
-    pw_error(tmp_file, 1, 1);
-  if (begin.st_mtime == end.st_mtime) {
-    (void)fprintf(stderr, "%s: no changes made\n", progname);
-    pw_error((char *)NULL, 0, 0);
+  if (program == VIGR) {
+    strncpy(orig_file, SGROUP_FILE, FILENAMELEN-1);
+    strncpy(tmp_file, SGTMP_FILE, FILENAMELEN-1);
+    strncpy(tmptmp_file, SGTMPTMP_FILE, FILENAMELEN-1);
+  } else {
+    strncpy(orig_file, SHADOW_FILE, FILENAMELEN-1);
+    strncpy(tmp_file, SPTMP_FILE, FILENAMELEN-1);
+    strncpy(tmptmp_file, SPTMPTMP_FILE, FILENAMELEN-1);
   }
-  pw_unlock();
+
+  if (!access(orig_file, X_OK)) {
+    char response[80];
+
+    printf((program == VIGR)
+	   ? _("You are using shadow groups on this system.\n")
+	   : _("You are using shadow passwords on this system.\n"));
+    printf(_("Would you like to edit %s now [y/n]? "), orig_file);
+
+    /* EOF means no */
+    if (fgets(response, sizeof(response), stdin)) {
+	if (response[0] == 'y' || response[0] == 'Y')
+	    edit_file();
+    }
+  }
+
   exit(0);
 }
