@@ -34,6 +34,12 @@
  *  oct 5 1994 -- almost entirely re-written to allow for process names.
  *  modifications (c) salvatore valente <svalente@mit.edu>
  *  may be used / modified / distributed under the same terms as the original.
+ *
+ *  1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+ *  - added Native Language Support
+ *
+ *  1999-11-13 aeb Accept signal numers 128+s.
+ *
  */
 
 #include <stdio.h>
@@ -42,26 +48,99 @@
 #include <ctype.h>
 #include <unistd.h>
 #include <signal.h>
+#include "kill.h"
+#include "nls.h"
 
-#ifdef __linux__
-/*
- *  sys_signame -- an ordered list of signals.
- *  lifted from /usr/include/linux/signal.h
- *  this particular order is only correct for linux.
- *  this is _not_ portable.
- */
-char *sys_signame[NSIG] = {
-    "zero",  "HUP",  "INT",   "QUIT", "ILL",   "TRAP", "IOT",  "UNUSED",
-    "FPE",   "KILL", "USR1",  "SEGV", "USR2",  "PIPE", "ALRM", "TERM",
-    "STKFLT","CHLD", "CONT",  "STOP", "TSTP",  "TTIN", "TTOU", "IO",
-    "XCPU",  "XFSZ", "VTALRM","PROF", "WINCH", NULL
-};
+#define SIZE(a)	(sizeof(a)/sizeof(a[0]))
+
+struct signv {
+	char *name;
+	int val;
+} sys_signame[] = {
+	/* POSIX signals */
+	{ "HUP",	SIGHUP },	/* 1 */
+	{ "INT",	SIGINT }, 	/* 2 */
+	{ "QUIT",	SIGQUIT }, 	/* 3 */
+	{ "ILL",	SIGILL }, 	/* 4 */
+	{ "ABRT",	SIGABRT }, 	/* 6 */
+	{ "FPE",	SIGFPE }, 	/* 8 */
+	{ "KILL",	SIGKILL }, 	/* 9 */
+	{ "SEGV",	SIGSEGV }, 	/* 11 */
+	{ "PIPE",	SIGPIPE }, 	/* 13 */
+	{ "ALRM",	SIGALRM }, 	/* 14 */
+	{ "TERM",	SIGTERM }, 	/* 15 */
+	{ "USR1",	SIGUSR1 }, 	/* 10 (arm,i386,m68k,ppc), 30 (alpha,sparc*), 16 (mips) */
+	{ "USR2",	SIGUSR2 }, 	/* 12 (arm,i386,m68k,ppc), 31 (alpha,sparc*), 17 (mips) */
+	{ "CHLD",	SIGCHLD }, 	/* 17 (arm,i386,m68k,ppc), 20 (alpha,sparc*), 18 (mips) */
+	{ "CONT",	SIGCONT }, 	/* 18 (arm,i386,m68k,ppc), 19 (alpha,sparc*), 25 (mips) */
+	{ "STOP",	SIGSTOP },	/* 19 (arm,i386,m68k,ppc), 17 (alpha,sparc*), 23 (mips) */
+	{ "TSTP",	SIGTSTP },	/* 20 (arm,i386,m68k,ppc), 18 (alpha,sparc*), 24 (mips) */
+	{ "TTIN",	SIGTTIN },	/* 21 (arm,i386,m68k,ppc,alpha,sparc*), 26 (mips) */
+	{ "TTOU",	SIGTTOU },	/* 22 (arm,i386,m68k,ppc,alpha,sparc*), 27 (mips) */
+	/* Miscellaneous other signals */
+#ifdef SIGTRAP
+	{ "TRAP",	SIGTRAP },	/* 5 */
 #endif
+#ifdef SIGIOT
+	{ "IOT",	SIGIOT }, 	/* 6, same as SIGABRT */
+#endif
+#ifdef SIGEMT
+	{ "EMT",	SIGEMT }, 	/* 7 (mips,alpha,sparc*) */
+#endif
+#ifdef SIGBUS
+	{ "BUS",	SIGBUS },	/* 7 (arm,i386,m68k,ppc), 10 (mips,alpha,sparc*) */
+#endif
+#ifdef SIGSYS
+	{ "SYS",	SIGSYS }, 	/* 12 (mips,alpha,sparc*) */
+#endif
+#ifdef SIGSTKFLT
+	{ "STKFLT",	SIGSTKFLT },	/* 16 (arm,i386,m68k,ppc) */
+#endif
+#ifdef SIGURG
+	{ "URG",	SIGURG },	/* 23 (arm,i386,m68k,ppc), 16 (alpha,sparc*), 21 (mips) */
+#endif
+#ifdef SIGIO
+	{ "IO",		SIGIO },	/* 29 (arm,i386,m68k,ppc), 23 (alpha,sparc*), 22 (mips) */
+#endif
+#ifdef SIGPOLL
+	{ "POLL",	SIGPOLL },	/* same as SIGIO */
+#endif
+#ifdef SIGCLD
+	{ "CLD",	SIGCLD },	/* same as SIGCHLD (mips) */
+#endif
+#ifdef SIGXCPU
+	{ "XCPU",	SIGXCPU },	/* 24 (arm,i386,m68k,ppc,alpha,sparc*), 30 (mips) */
+#endif
+#ifdef SIGXFSZ
+	{ "XFSZ",	SIGXFSZ },	/* 25 (arm,i386,m68k,ppc,alpha,sparc*), 31 (mips) */
+#endif
+#ifdef SIGVTALRM
+	{ "VTALRM",	SIGVTALRM },	/* 26 (arm,i386,m68k,ppc,alpha,sparc*), 28 (mips) */
+#endif
+#ifdef SIGPROF
+	{ "PROF",	SIGPROF },	/* 27 (arm,i386,m68k,ppc,alpha,sparc*), 29 (mips) */
+#endif
+#ifdef SIGPWR
+	{ "PWR",	SIGPWR },	/* 30 (arm,i386,m68k,ppc), 29 (alpha,sparc*), 19 (mips) */
+#endif
+#ifdef SIGINFO
+	{ "INFO",	SIGINFO },	/* 29 (alpha) */
+#endif
+#ifdef SIGLOST
+	{ "LOST",	SIGLOST }, 	/* 29 (arm,i386,m68k,ppc,sparc*) */
+#endif
+#ifdef SIGWINCH
+	{ "WINCH",	SIGWINCH },	/* 28 (arm,i386,m68k,ppc,alpha,sparc*), 20 (mips) */
+#endif
+#ifdef SIGUNUSED
+	{ "UNUSED",	SIGUNUSED },	/* 31 (arm,i386,m68k,ppc) */
+#endif
+};
 
 int main (int argc, char *argv[]);
 extern char *mybasename(char *);
 int signame_to_signum (char *sig);
-int arg_to_signum (char *arg);
+int arg_to_signum (char *arg, int mask);
 void nosig (char *name);
 void printsig (int sig);
 void printsignals (FILE *fp);
@@ -70,19 +149,25 @@ int kill_verbose (char *procname, int pid, int sig);
 
 extern int *get_pids (char *, int);
 
-char version_string[] = "kill v2.0\n";
-char *whoami;
+static char *progname;
 
 int main (int argc, char *argv[])
 {
     int errors, numsig, pid;
-    char *ep, *arg;
+    char *ep, *arg, *p;
     int do_pid, do_kill, check_all;
     int *pids, *ip;
 
-    whoami = mybasename (*argv);
+    progname = argv[0];
+    if ((p = strrchr(progname, '/')) != NULL)
+	    progname = p+1;
+
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
+
     numsig = SIGTERM;
-    do_pid = (! strcmp (whoami, "pid"));
+    do_pid = (! strcmp (progname, "pid")); 	/* Yecch */
     do_kill = 0;
     check_all = 0;
 
@@ -94,11 +179,13 @@ int main (int argc, char *argv[])
 	if (*arg != '-') {
 	    break;
 	}
-	if (! strcmp (arg, "-u")) {
-	    return usage (0);
+	if (! strcmp (arg, "--")) {
+	    argc--, argv++;
+	    break;
 	}
-	if (! strcmp (arg, "-v")) {
-	    fputs (version_string, stdout);
+	if (! strcmp (arg, "-v") || ! strcmp (arg, "-V") ||
+	    ! strcmp (arg, "--version")) {
+	    printf(_("%s from %s\n"), progname, util_linux_version);
 	    return 0;
 	}
 	if (! strcmp (arg, "-a")) {
@@ -113,10 +200,10 @@ int main (int argc, char *argv[])
 	    if (argc > 2) {
 		return usage (1);
 	    }
-	    /* argc == 2 */
+	    /* argc == 2, accept "kill -l $?" */
 	    arg = argv[1];
-	    if ((numsig = arg_to_signum (arg)) < 0) {
-		fprintf (stderr, "%s: unknown signal %s\n", whoami, arg);
+	    if ((numsig = arg_to_signum (arg, 1)) < 0) {
+		fprintf (stderr, _("%s: unknown signal %s\n"), progname, arg);
 		return 1;
 	    }
 	    printsig (numsig);
@@ -137,7 +224,7 @@ int main (int argc, char *argv[])
 		return usage (1);
 	    argc--, argv++;
 	    arg = *argv;
-	    if ((numsig = arg_to_signum (arg)) < 0) {
+	    if ((numsig = arg_to_signum (arg, 0)) < 0) {
 		nosig (arg);
 		return 1;
 	    }
@@ -147,11 +234,12 @@ int main (int argc, char *argv[])
 	    so it's probably something like -HUP, or -1/-n
 	    try to deal with it.
 	    -n could be signal n, or pid -n (i.e. process group n).
+	    In case of doubt POSIX tells us to assume a signal.
 	    If a signal has been parsed, assume it's a pid, break */
 	if (do_kill)
 	  break;
 	arg++;
-	if ((numsig = arg_to_signum (arg)) < 0) {
+	if ((numsig = arg_to_signum (arg, 0)) < 0) {
 	    return usage (1);
 	}
 	do_kill++;
@@ -178,8 +266,8 @@ int main (int argc, char *argv[])
 	    pids = get_pids (arg, check_all);
 	    if (! pids) {
 		errors++;
-		fprintf (stderr, "%s: can't find process \"%s\"\n",
-			 whoami, arg);
+		fprintf (stderr, _("%s: can't find process \"%s\"\n"),
+			 progname, arg);
 		continue;
 	    }
 	    for (ip = pids; *ip >= 0; ip++)
@@ -197,20 +285,22 @@ int signame_to_signum (char *sig)
 
     if (! strncasecmp (sig, "sig", 3))
 	sig += 3;
-    for (n = 1; (n < NSIG) && (sys_signame[n] != NULL); n++) {
-	if (! strcasecmp (sys_signame[n], sig))
-	    return n;
+    for (n = 0; n < SIZE(sys_signame); n++) {
+	if (! strcasecmp (sys_signame[n].name, sig))
+	    return sys_signame[n].val;
     }
     return (-1);
 }
 
-int arg_to_signum (char *arg)
+int arg_to_signum (char *arg, int maskbit)
 {
     int numsig;
     char *ep;
 
     if (isdigit (*arg)) {
 	numsig = strtol (arg, &ep, 10);
+	if (numsig >= NSIG && maskbit && (numsig & 128) != 0)
+	    numsig -= 128;
 	if (*ep != 0 || numsig < 0 || numsig >= NSIG)
 	    return (-1);
 	return (numsig);
@@ -220,28 +310,39 @@ int arg_to_signum (char *arg)
 
 void nosig (char *name)
 {
-    fprintf (stderr, "%s: unknown signal %s; valid signals:\n", whoami, name);
+    fprintf (stderr, _("%s: unknown signal %s; valid signals:\n"), progname, name);
     printsignals (stderr);
 }
 
 void printsig (int sig)
 {
-    printf ("%s\n", sys_signame[sig]);
+    int n;
+
+    for (n = 0; n < SIZE(sys_signame); n++) {
+	if (sys_signame[n].val == sig) {
+	    printf ("%s\n", sys_signame[n].name);
+	    return;
+	}
+    }
+    printf("%d\n", sig);
 }
 
 void printsignals (FILE *fp)
 {
-    int n;
+    int n, lth;
+    int lpos = 0;
 
-    for (n = 1; (n < NSIG) && (sys_signame[n] != NULL); n++) {
-	fputs (sys_signame[n], fp);
-	if (n == (NSIG / 2) || n == (NSIG - 1))
+    for (n = 0; n < SIZE(sys_signame); n++) {
+	lth = 1+strlen(sys_signame[n].name);
+	if (lpos+lth > 72) {
 	    fputc ('\n', fp);
-	else
+	    lpos = 0;
+	} else if (lpos)
 	    fputc (' ', fp);
+	lpos += lth;
+	fputs (sys_signame[n].name, fp);
     }
-    if (n < (NSIG - 1))
-	fputc ('\n', fp);
+    fputc ('\n', fp);
 }
 
 int usage (int status)
@@ -249,8 +350,8 @@ int usage (int status)
     FILE *fp;
 
     fp = (status == 0 ? stdout : stderr);
-    fprintf (fp, "usage: %s [ -s signal | -p ] [ -a ] pid ...\n", whoami);
-    fprintf (fp, "       %s -l [ signal ]\n", whoami);
+    fprintf (fp, _("usage: %s [ -s signal | -p ] [ -a ] pid ...\n"), progname);
+    fprintf (fp, _("       %s -l [ signal ]\n"), progname);
     return status;
 }
 
@@ -261,7 +362,7 @@ int kill_verbose (char *procname, int pid, int sig)
 	return 0;
     }
     if (kill (pid, sig) < 0) {
-	fprintf (stderr, "%s ", whoami);
+	fprintf (stderr, "%s ", progname);
 	perror (procname);
 	return 1;
     }
