@@ -39,25 +39,22 @@ the maximum number of symbolic links this system can have.
 The program exits with a 1 status ONLY if it finds it cannot
 chdir to /,  or if it encounters an unknown file type.
 
--------------------------------------------------------------*/
+1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+- added Native Language Support
 
-#ifndef lint
-static char *RCSid = "$Id: namei.c,v 1.6 1997/07/06 00:13:09 aebr Exp $";
-#endif
+-------------------------------------------------------------*/
 
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
+#include <stdlib.h>
+#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/param.h>
+#include "nls.h"
 
-#ifndef __GNU_LIBRARY__
-extern char *sys_errlist[];
-#endif
-
-extern int errno;
-#define ERR	sys_errlist[errno],errno
+#define ERR	strerror(errno),errno
 
 int symcount;
 int mflag = 0;
@@ -67,20 +64,20 @@ int xflag = 0;
 #define MAXSYMLINKS 256
 #endif
 
-static char *pperm();
+static char *pperm(unsigned short);
+static void namei(char *, int);
+static void usage(void);
 
 int
-main(argc, argv)
-int argc;
-char *argv[];
-{
-    void namei(), usage();
-    char *getwd();
-    int getopt();
+main(int argc, char **argv) {
     extern int optind;
-    register int c;
+    int c;
     char curdir[MAXPATHLEN];
 
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
+    
     if(argc < 2)
 	usage();
 
@@ -100,8 +97,8 @@ char *argv[];
 	}
     }
 
-    if(getwd(curdir) == NULL){
-	(void)fprintf(stderr, "namei: unable to get current directory - %s\n", curdir);
+    if(getcwd(curdir, sizeof(curdir)) == NULL){
+	(void)fprintf(stderr, _("namei: unable to get current directory - %s\n"), curdir);
 	exit(1);
     }
 
@@ -112,17 +109,16 @@ char *argv[];
 	namei(argv[optind], 0);
 
 	if(chdir(curdir) == -1){
-	    (void)fprintf(stderr, "namei: unable to chdir to %s - %s (%d)\n", curdir, ERR);
+	    (void)fprintf(stderr, _("namei: unable to chdir to %s - %s (%d)\n"), curdir, ERR);
 	    exit(1);
 	}
     }
     return 0;
 }
 
-void
-usage()
-{
-    (void)fprintf(stderr,"usage: namei [-mx] pathname [pathname ...]\n");
+static void
+usage(void) {
+    (void)fprintf(stderr,_("usage: namei [-mx] pathname [pathname ...]\n"));
     exit(1);
 }
 
@@ -130,16 +126,12 @@ usage()
 #define NODEV		(dev_t)(-1)
 #endif
 
-void
-namei(file, lev)
-
-register char *file;
-register int lev;
-{
-    register char *cp;
+static void
+namei(char *file, int lev) {
+    char *cp;
     char buf[BUFSIZ], sym[BUFSIZ];
     struct stat stb;
-    register int i;
+    int i;
     dev_t lastdev = NODEV;
 
     /*
@@ -151,14 +143,14 @@ register int lev;
 	    file++;
 	
 	if(chdir("/") == -1){
-	    (void)fprintf(stderr,"namei: could not chdir to root!\n");
+	    (void)fprintf(stderr,_("namei: could not chdir to root!\n"));
 	    exit(1);
 	}
 	for(i = 0; i < lev; i++)
 	    (void)printf("  ");
 
 	if(stat("/", &stb) == -1){
-	    (void)fprintf(stderr, "namei: could not stat root!\n");
+	    (void)fprintf(stderr, _("namei: could not stat root!\n"));
 	    exit(1);
 	}
 	lastdev = stb.st_dev;
@@ -213,7 +205,7 @@ register int lev;
 		 */
 		
 		if(chdir(buf) == -1){
-		    (void)printf(" ? could not chdir into %s - %s (%d)\n", buf, ERR );
+		    (void)printf(_(" ? could not chdir into %s - %s (%d)\n"), buf, ERR );
 		    return;
 		}
 		if(xflag && lastdev != stb.st_dev && lastdev != NODEV){
@@ -242,7 +234,7 @@ register int lev;
 		
 		bzero(sym, BUFSIZ);
 		if(readlink(buf, sym, BUFSIZ) == -1){
-		    (void)printf(" ? problems reading symlink %s - %s (%d)\n", buf, ERR);
+		    (void)printf(_(" ? problems reading symlink %s - %s (%d)\n"), buf, ERR);
 		    return;
 		}
 
@@ -252,11 +244,12 @@ register int lev;
 		    (void)printf(" l %s -> %s", buf, sym);
 
 		if(symcount > 0 && symcount++ > MAXSYMLINKS){
-		    (void)printf("  *** EXCEEDED UNIX LIMIT OF SYMLINKS ***");
+		    (void)printf(_("  *** EXCEEDED UNIX LIMIT OF SYMLINKS ***\n"));
 		    symcount = -1;
+		} else {
+		    (void)printf("\n");
+		    namei(sym, lev + 1);
 		}
-		(void)printf("\n");
-		namei(sym, lev + 1);
 		break;
 
 	    case S_IFCHR:
@@ -288,7 +281,7 @@ register int lev;
 		break;
 		
 	    default:
-		(void)fprintf(stderr,"namei: unknown file type 0%06o on file %s\n", stb.st_mode, buf );
+		(void)fprintf(stderr,_("namei: unknown file type 0%06o on file %s\n"), stb.st_mode, buf );
 		exit(1);
 	    
 	}
@@ -301,9 +294,7 @@ register int lev;
  * For example 0755 produces "rwxr-xr-x"
  */
 static char *
-pperm(mode)
-unsigned short mode;
-{
+pperm(unsigned short mode) {
 	unsigned short m;
 	static char buf[16];
 	char *bp;
