@@ -33,12 +33,18 @@
 
 /* *:aeb */
 
+/* 1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+ * - added Native Language Support
+ */
+
 #include <sys/param.h>
 #include <sys/dir.h>
 #include <sys/stat.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include "nls.h"
 
 void zerof(void);
 void getlist(int *, char ***, char ***, int *);
@@ -99,11 +105,14 @@ static char *bindirs[] = {
    "/usr/libexec",
    "/usr/share",
 
+   "/opt/*/bin",
+
 	0
 };
 
 static char *mandirs[] = {
 	"/usr/man/*",
+	"/usr/share/man/*",
 	"/usr/X386/man/*",
 	"/usr/X11/man/*",
 	"/usr/TeX/man/*",
@@ -136,15 +145,16 @@ char	uflag;
  * look for source, documentation and binaries
  */
 int
-main(argc, argv)
-	int argc;
-	char *argv[];
-{
+main(int argc, char **argv) {
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+	
 
 	argc--, argv++;
 	if (argc == 0) {
 usage:
-		fprintf(stderr, "whereis [ -sbmu ] [ -SBM dir ... -f ] name...\n");
+		fprintf(stderr, _("whereis [ -sbmu ] [ -SBM dir ... -f ] name...\n"));
 		exit(1);
 	}
 	do
@@ -326,6 +336,9 @@ findin(char *dir, char *cp) {
 		if (dirp == NULL)
 			return;
 		while ((dp = readdir(dirp)) != NULL) {
+			if (!strcmp(dp->d_name, ".") ||
+			    !strcmp(dp->d_name, ".."))
+				continue;
 			if (strlen(dp->d_name) + l > sizeof(dirbuf))
 				continue;
 			sprintf(d, "%s", dp->d_name);
