@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1980 Regents Of the University of California.
+ * Copyright (C) 1980 The Regents of the University of California.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -26,12 +26,23 @@
 **	  on linux/axp.
 **	modified by Kars de Jong <jongk@cs.utwente.nl> to use terminfo instead
 **	  of termcap.
+	1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+	- added Native Language Support
+	1999-03-19 Arnaldo Carvalho de Melo <acme@conectiva.com.br>
+	- more nls translatable strings
+	1999-05-09 aeb - applied a RedHat patch (setjmp->sigsetjmp); without it
+	a second ^Z would fail.
+	1999-05-09 aeb - undone Kars' work, so that more works without
+	libcurses (and hence can be in /bin with libcurses being in /usr/lib
+	which may not be mounted). However, when termcap is not present curses
+	can still be used.
 */
 
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>		/* for alloca() */
+#include <stdarg.h>		/* for va_start() etc */
 #include <sys/param.h>
 #include <ctype.h>
 #include <signal.h>
@@ -43,15 +54,16 @@
 #include <sys/file.h>
 #include <sys/wait.h>
 #include <a.out.h>
-#if NCH
-#include <ncurses.h>
-#else
-#include <curses.h>
-#endif
-#include <term.h>
+#include <locale.h>
+#include "xstrncpy.h"
+#include "nls.h"
 
-#define HELPFILE	"/usr/lib/more.help"
-#define VI		"/usr/bin/vi"
+#define _REGEX_RE_COMP
+#include <regex.h>
+#undef _REGEX_RE_COMP
+
+/* #define MOREHELPFILE "/usr/lib/more.help" */
+#define VI		"vi"	/* found on the user's path */
 
 #define Fopen(s,m)	(Currline = 0,file_pos=0,fopen(s,m))
 #define Ftell(f)	file_pos
@@ -71,8 +83,8 @@ void home(void);
 void error (char *mess);
 void do_shell (char *filename);
 int  colon (char *filename, int cmd, int nlines);
-int  expand (char *outbuf, char *inbuf);
-void argscan(char *s);
+int  expand (char **outbuf, char *inbuf);
+void argscan(char *s,char *argv0);
 void rdline (register FILE *f);
 void copy_file(register FILE *f);
 void search(char buf[], FILE *file, register int n);
@@ -93,6 +105,9 @@ int  get_line(register FILE *f, int *length);
 void prbuf (register char *s, register int n);
 int  xprintf (char *fmt, ...);
 void execute (char *filename, char *cmd, ...);
+void errwrite (char *txt);
+void errwrite1 (char *sym);
+FILE *checkf (char *, int *);
 
 #define TBUFSIZ	1024
 #define LINSIZ	256
@@ -105,7 +120,7 @@ struct termios	otty, savetty0;
 long		file_pos, file_size;
 int		fnum, no_intty, no_tty, slow_tty;
 int		dum_opt, dlines;
-void		onquit(), onsusp(), chgwinsz(), end_it();
+void		onquit(int), onsusp(int), chgwinsz(int), end_it(int);
 int		nscroll = 11;	/* Number of lines scrolled by 'd' */
 int		fold_opt = 1;	/* Fold long lines */
 int		stop_opt = 1;	/* Stop after form feeds */
@@ -128,9 +143,8 @@ char		**fnames;	/* The list of file names */
 int		nfiles;		/* Number of files left to process */
 char		*shell;		/* The name of the shell to use */
 int		shellp;		/* A previous shell command exists */
-char		ch;
-jmp_buf		restore;
-char		Line[LINSIZ];	/* Line buffer */
+sigjmp_buf	restore;
+char		Line[LINSIZ+2];	/* Line buffer */
 int		Lpp = 24;	/* lines per page */
 char		*Clear;		/* clear screen */
 char		*eraseln;	/* erase line */
@@ -142,30 +156,119 @@ char		*Home;		/* go to home */
 char		*cursorm;	/* cursor movement */
 char		cursorhome[40];	/* contains cursor movement to home */
 char		*EodClr;	/* clear rest of screen */
-char		*tgetstr();
 int		Mcol = 80;	/* number of columns */
 int		Wrap = 1;	/* set if automargins */
 int		soglitch;	/* terminal has standout mode glitch */
 int		ulglitch;	/* terminal has underline mode glitch */
 int		pstate = 0;	/* current UL state */
-char		*getenv();
-static		magic();
+static int	magic(FILE *, char *);
 struct {
     long chrctr, line;
 } context, screen_start;
-/* extern */ char	PC;		/* pad character */
+extern char	PC;		/* pad character */
 
-void
+#ifndef HAVE_termcap
+#define USE_CURSES
+#endif
+
+#ifdef USE_CURSES
+
+#if NCH
+#include <ncurses.h>
+#else
+#include <curses.h>
+#endif
+#include <term.h>			/* include after <curses.h> */
+
+static void
+my_putstring(char *s) {
+     putp(s);
+}
+
+static void
+my_setupterm(const char *term, int fildes, int *errret) {
+     setupterm(term, fildes, errret);
+}
+
+static int
+my_tgetnum(char *s, char *ss) {
+     return tigetnum(ss);
+}
+
+static int
+my_tgetflag(char *s, char *ss) {
+     return tigetflag(ss);
+}
+
+static char *
+my_tgetstr(char *s, char *ss) {
+     return tigetstr(ss);
+}
+
+static char *
+my_tgoto(const char *cap, int col, int row) {
+     return tparm(cap, col, row);
+}
+
+#else /* no CURSES */
+#include <termcap.h>
+
+char termbuffer[4096];
+char tcbuffer[4096];
+char *strbuf = termbuffer;
+
+static void
+my_putstring(char *s) {
+     tputs (s, 1, putchar);
+}
+
+static void
+my_setupterm(const char *term, int fildes, int *errret) {
+     *errret = tgetent(tcbuffer, term);
+}
+
+static int
+my_tgetnum(char *s, char *ss) {
+     return tgetnum(s);
+}
+
+static int
+my_tgetflag(char *s, char *ss) {
+     return tgetflag(s);
+}
+
+static char *
+my_tgetstr(char *s, char *ss) {
+     return tgetstr(s, &strbuf);
+}
+
+static char *
+my_tgoto(const char *cap, int col, int row) {
+     return tgoto(cap, col, row);
+}
+
+
+#endif /* USE_CURSES */
+
+static void
 idummy(int *kk) {}
 
-void
+static void
 Fdummy(FILE **ff) {}
 
+static void
+usage(char *s) {
+	char *p = strrchr(s, '/');
+	fprintf(stderr,
+		_("usage: %s [-dflpcsu] [+linenum | +/pattern] name1 name2 ...\n"),
+		p ? p + 1 : s);
+}
+
 int main(int argc, char **argv) {
     FILE	*f;
     char	*s;
     char	*p;
-    char	ch;
+    int		ch;
     int		left;
     int		prnames = 0;
     int		initopt = 0;
@@ -173,8 +276,11 @@ int main(int argc, char **argv) {
     int		clearit = 0;
     int		initline = 0;
     char	initbuf[80];
-    FILE	*checkf();
 
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
+    
     /* avoid gcc complaints about register variables that
        may be clobbered by a longjmp, by forcing our variables here
        to be non-register */
@@ -183,14 +289,15 @@ int main(int argc, char **argv) {
 
     nfiles = argc;
     fnames = argv;
+    setlocale(LC_ALL, "");
     initterm ();
     nscroll = Lpp/2 - 1;
     if (nscroll <= 0)
 	nscroll = 1;
-    if((s = getenv("MORE")) != NULL) argscan(s);
+    if((s = getenv("MORE")) != NULL) argscan(s,argv[0]);
     while (--nfiles > 0) {
 	if ((ch = (*++fnames)[0]) == '-') {
-	    argscan(*fnames+1);
+	    argscan(*fnames+1,argv[0]);
 	}
 	else if (ch == '+') {
 	    s = *fnames;
@@ -227,12 +334,7 @@ int main(int argc, char **argv) {
     if (nfiles > 1)
 	prnames++;
     if (!no_intty && nfiles == 0) {
-	char *rindex();
-
-	p = rindex(argv[0], '/');
-	fputs("usage: ",stderr);
-	fputs(p ? p + 1 : argv[0],stderr);
-	fputs(" [-dfln] [+linenum | +/pattern] name1 name2 ...\n",stderr);
+	usage(argv[0]);
 	exit(1);
     }
     else
@@ -283,11 +385,10 @@ int main(int argc, char **argv) {
 	if ((f = checkf (fnames[fnum], &clearit)) != NULL) {
 	    context.line = context.chrctr = 0;
 	    Currline = 0;
-	    if (firstf) setjmp (restore);
+	    if (firstf) sigsetjmp (restore, 1);
 	    if (firstf) {
 		firstf = 0;
-		if (srchopt)
-		{
+		if (srchopt) {
 		    search (initbuf, f, 1);
 		    if (noscroll)
 			left--;
@@ -296,15 +397,16 @@ int main(int argc, char **argv) {
 		    skiplns (initline, f);
 	    }
 	    else if (fnum < nfiles && !no_tty) {
-		setjmp (restore);
+		sigsetjmp (restore, 1);
 		left = command (fnames[fnum], f);
 	    }
 	    if (left != 0) {
-		if ((noscroll || clearit) && (file_size != LONG_MAX))
+		if ((noscroll || clearit) && (file_size != LONG_MAX)) {
 		    if (clreol)
 			home ();
 		    else
 			doclear ();
+		}
 		if (prnames) {
 		    if (bad_so)
 			erasep (0);
@@ -329,7 +431,7 @@ int main(int argc, char **argv) {
 		    within = 0;
 		}
 	    }
-	    setjmp (restore);
+	    sigsetjmp (restore, 1);
 	    fflush(stdout);
 	    fclose(f);
 	    screen_start.line = screen_start.chrctr = 0L;
@@ -342,7 +444,7 @@ int main(int argc, char **argv) {
     exit(0);
 }
 
-void argscan(char *s) {
+void argscan(char *s, char *argv0) {
 	int seen_num = 0;
 
 	while (*s != '\0') {
@@ -378,6 +480,16 @@ void argscan(char *s) {
 		  case 'u':
 			ul_opt = 0;
 			break;
+		  case '-': case ' ': case '\t':
+			break;
+		  default:
+			fputs(argv0,stderr);
+			fputs(": unknown option \"-",stderr);
+			putc(*s,stderr);
+			fputs("\"\n",stderr);
+			usage(argv0);
+			exit(1);
+			break;
 		}
 		s++;
 	}
@@ -406,7 +518,7 @@ checkf (fs, clearfirst)
 		return((FILE *)NULL);
 	}
 	if ((stbuf.st_mode & S_IFMT) == S_IFDIR) {
-		xprintf("\n*** %s: directory ***\n\n", fs);
+		xprintf(_("\n*** %s: directory ***\n\n"), fs);
 		return((FILE *)NULL);
 	}
 	if ((f = Fopen(fs, "r")) == NULL) {
@@ -434,21 +546,23 @@ magic(f, fs)
 	FILE *f;
 	char *fs;
 {
-	struct exec ex;
+	char twobytes[2];
 
-	if (fread(&ex, sizeof(ex), 1, f) == 1)
-		switch(ex.a_info) {
-		case OMAGIC:
-		case NMAGIC:
-		case ZMAGIC:
+	if (fread(twobytes, 2, 1, f) == 1) {
+		switch(twobytes[0] + (twobytes[1]<<8)) {
+		case OMAGIC:	/* 0407 */
+		case NMAGIC:	/* 0410 */
+		case ZMAGIC:	/* 0413 */
 		case 0405:
 		case 0411:
 		case 0177545:
-			xprintf("\n******** %s: Not a text file ********\n\n", fs);
+		case 0x457f:		/* simple ELF detection */
+			xprintf(_("\n******** %s: Not a text file ********\n\n"), fs);
 			(void)fclose(f);
 			return(1);
 		}
-	(void)fseek(f, 0L, L_SET);		/* rewind() not necessary */
+	}
+	(void)fseek(f, 0L, SEEK_SET);		/* rewind() not necessary */
 	return(0);
 }
 
@@ -498,7 +612,7 @@ void screen (register FILE *f, register int num_lines)
 	    num_lines--;
 	}
 	if (pstate) {
-		putp (ULexit);
+		my_putstring (ULexit);
 		pstate = 0;
 	}
 	fflush(stdout);
@@ -512,7 +626,7 @@ void screen (register FILE *f, register int num_lines)
 	if (Pause && clreol)
 	    clreos ();
 	Ungetc (c, f);
-	setjmp (restore);
+	sigsetjmp (restore, 1);
 	Pause = 0; startup = 0;
 	if ((num_lines = command (NULL, f)) == 0)
 	    return;
@@ -534,21 +648,21 @@ void screen (register FILE *f, register int num_lines)
 ** Come here if a quit signal is received
 */
 
-void onquit()
-{
+void onquit(int dummy) {
     signal(SIGQUIT, SIG_IGN);
     if (!inwait) {
 	putchar ('\n');
 	if (!startup) {
 	    signal(SIGQUIT, onquit);
-	    longjmp (restore, 1);
+	    siglongjmp (restore, 1);
 	}
 	else
 	    Pause++;
     }
     else if (!dum_opt && notell) {
-	write (2, "[Use q or Q to quit]", 20);
-	promptlen += 20;
+	char *s = _("[Use q or Q to quit]");
+	errwrite(s);
+	promptlen += strlen(s);
 	notell = 0;
     }
     signal(SIGQUIT, onquit);
@@ -559,8 +673,7 @@ void onquit()
 */
 
 #ifdef SIGWINCH
-void chgwinsz()
-{
+void chgwinsz(int dummy) {
     struct winsize win;
 
     (void) signal(SIGWINCH, SIG_IGN);
@@ -583,9 +696,7 @@ void chgwinsz()
 ** Clean up terminal state and exit. Also come here if interrupt signal received
 */
 
-void end_it ()
-{
-
+void end_it (int dummy) {
     reset_tty ();
     if (clreol) {
 	putchar ('\r');
@@ -597,7 +708,7 @@ void end_it ()
 	fflush (stdout);
     }
     else
-	write (2, "\n", 1);
+	errwrite("\n");
     _exit(0);
 }
 
@@ -610,11 +721,10 @@ void copy_file(register FILE *f) {
 
 /* Simplified printf function */
 
-int xprintf (char *fmt, ...)
-{
+int xprintf (char *fmt, ...) {
 	va_list ap;
-	register char ch;
-	register int ccount;
+	char ch;
+	int ccount;
 
 	ccount = 0;
 	va_start(ap, fmt);
@@ -667,7 +777,7 @@ int printd (int n)
 /* Put the print representation of an integer into a string */
 static char *sptr;
 
-void Sprintf (int n) {
+static void Sprintf (int n) {
     int a;
 
     if ((a = n/10) != 0)
@@ -675,14 +785,14 @@ void Sprintf (int n) {
     *sptr++ = n % 10 + '0';
 }
 
-void scanstr (int n, char *str)
+static void scanstr (int n, char *str)
 {
     sptr = str;
     Sprintf (n);
     *sptr = '\0';
 }
 
-static char my_bell = ctrl('G');
+#define ringbell()	errwrite("\007");
 
 #ifdef undef
 strlen (s)
@@ -701,15 +811,13 @@ char *s;
 ** string "string"
 */
 
-int tailequ (path, string)
-char *path;
-register char *string;
+static int tailequ (char *path, register char *string)
 {
 	register char *tail;
 
 	tail = path + strlen(path);
-	while (tail >= path)
-		if (*(--tail) == '/')
+	while (--tail >= path)
+		if (*tail == '/')
 			break;
 	++tail;
 	while (*tail++ == *string++)
@@ -718,39 +826,38 @@ register char *string;
 	return(0);
 }
 
-void prompt (filename)
-char *filename;
+static void prompt (char *filename)
 {
     if (clreol)
 	cleareol ();
     else if (promptlen > 0)
 	kill_line ();
     if (!hard) {
-	promptlen = 8;
+	promptlen = 0;
 	if (Senter && Sexit) {
-	    putp (Senter);
+	    my_putstring (Senter);
 	    promptlen += (2 * soglitch);
 	}
 	if (clreol)
 	    cleareol ();
-	pr("--More--");
+	promptlen += pr(_("--More--"));
 	if (filename != NULL) {
-	    promptlen += xprintf ("(Next file: %s)", filename);
-	}
-	else if (!no_intty) {
-	    promptlen += xprintf ("(%d%%)", (int)((file_pos * 100) / file_size));
+	    promptlen += xprintf (_("(Next file: %s)"), filename);
+	} else if (!no_intty) {
+	    promptlen += xprintf ("(%d%%)",
+				  (int)((file_pos * 100) / file_size));
 	}
 	if (dum_opt) {
-	    promptlen += pr("[Press space to continue, 'q' to quit.]");
+	    promptlen += pr(_("[Press space to continue, 'q' to quit.]"));
 	}
 	if (Senter && Sexit)
-	    putp (Sexit);
+	    my_putstring (Sexit);
 	if (clreol)
 	    clreos ();
 	fflush(stdout);
     }
     else
-	write (2, &my_bell, 1);
+	ringbell();
     inwait++;
 }
 
@@ -787,11 +894,25 @@ int get_line(register FILE *f, int *length)
 	    break;
 	}
 	*p++ = c;
-	if (c == '\t')
+#if 0
+	if (c == '\033') {      /* ESC */
+		c = Getc(f);
+		while (c > ' ' && c < '0' && p < &Line[LINSIZ - 1]) {
+			*p++ = c;
+			c = Getc(f);
+		}
+		if (c >= '0' && c < '\177' && p < &Line[LINSIZ - 1]) {
+			*p++ = c;
+			c = Getc(f);
+			continue;
+		}
+	}
+#endif
+	if (c == '\t') {
 	    if (!hardtabs || (column < promptlen && !hard)) {
 		if (hardtabs && eraseln && !dumb) {
 		    column = 1 + (column | 7);
-		    putp (eraseln);
+		    my_putstring (eraseln);
 		    promptlen = 0;
 		}
 		else {
@@ -805,9 +926,9 @@ int get_line(register FILE *f, int *length)
 	    }
 	    else
 		column = 1 + (column | 7);
-	else if (c == '\b' && column > 0)
+	} else if (c == '\b' && column > 0) {
 	    column--;
-	else if (c == '\r') {
+	} else if (c == '\r') {
 	    int next = Getc(f);
 	    if (next == '\n') {
 		p--;
@@ -816,20 +937,40 @@ int get_line(register FILE *f, int *length)
 	    }
 	    Ungetc(next,f);
 	    column = 0;
-	}
-	else if (c == '\f' && stop_opt) {
+	} else if (c == '\f' && stop_opt) {
 		p[-1] = '^';
 		*p++ = 'L';
 		column += 2;
 		Pause++;
-	}
-	else if (c == EOF) {
+	} else if (c == EOF) {
 	    *length = p - Line;
 	    return (column);
 	}
-	else if (c >= ' ' && c != RUBOUT)
-	    column++;
-	if (column >= Mcol && fold_opt) break;
+#if 0
+	else {
+		char ch[2]; /* for mblen() */
+
+		ch[0] = c;
+		ch[1] = '\0';
+		if (mblen(ch, 1) <= 0) {    /* multibyte, maybe 2 columns */
+			if (column == Mcol-1 && fold_opt) {
+				p--;
+				Ungetc(c, f);
+				break;
+			}
+			*p++ = Getc(f);
+			column += 2;
+		} else if (c & 0x80) {      /* maybe SJIS KANA, or utf-8 ? */
+			column++;
+		} else if (isprint(c))
+			column++;
+	}
+#else
+	else if (isprint(c))
+		column++;
+#endif
+	if (column >= Mcol && fold_opt)
+		break;
 	c = Getc (f);
     }
     if (column >= Mcol && Mcol > 0) {
@@ -862,7 +1003,7 @@ void erasep (register int col)
 	if (col == 0)
 	    putchar ('\r');
 	if (!dumb && eraseln)
-	    putp (eraseln);
+	    my_putstring (eraseln);
 	else
 	    for (col = promptlen - col; col > 0; col--)
 		putchar (' ');
@@ -886,12 +1027,12 @@ void kill_line ()
  */
 void cleareol()
 {
-    putp(eraseln);
+    my_putstring(eraseln);
 }
 
 void clreos()
 {
-    putp(EodClr);
+    my_putstring(EodClr);
 }
 
 /*
@@ -935,13 +1076,13 @@ void prbuf (register char *s, register int n)
 		if (c == ' ' && state == 0 && ulglitch && wouldul(s, n-1))
 		    state = 1;
 		else
-		    putp(state ? ULenter : ULexit);
+		    my_putstring(state ? ULenter : ULexit);
 	    }
 	    if (c != ' ' || pstate == 0 || state != 0 || ulglitch == 0)
 	        putchar(c);
 	    if (state && *chUL) {
 		pr(chBS);
-		putp(chUL);
+		my_putstring(chUL);
 	    }
 	    pstate = state;
 	}
@@ -954,7 +1095,7 @@ void
 doclear()
 {
     if (Clear && !hard) {
-	putp(Clear);
+	my_putstring(Clear);
 
 	/* Put out carriage return so that system doesn't
 	** get confused by escape sequences when expanding tabs
@@ -970,12 +1111,12 @@ doclear()
 void
 home()
 {
-    putp(Home);
+    my_putstring(Home);
 }
 
 static int lastcmd, lastarg, lastp;
 static int lastcolon;
-char shell_line[132];
+char shell_line[1000];
 
 /*
 ** Read a command and do it. A command consists of an optional integer
@@ -988,7 +1129,7 @@ int command (char *filename, register FILE *f)
 {
     register int nlines;
     register int retval = 0;
-    register char c;
+    register int c;
     char colonch;
     FILE *helpf;
     int done;
@@ -1013,7 +1154,7 @@ int command (char *filename, register FILE *f)
 	}
 	lastcmd = comchar;
 	lastarg = nlines;
-	if (comchar == otty.c_cc[VERASE]) {
+	if ((cc_t) comchar == otty.c_cc[VERASE]) {
 	    kill_line ();
 	    prompt (filename);
 	    continue;
@@ -1030,7 +1171,7 @@ int command (char *filename, register FILE *f)
 		register int initline;
 
 		if (no_intty) {
-		    write(2, &my_bell, 1);
+		    ringbell();
 		    return (-1);
 		}
 
@@ -1041,7 +1182,7 @@ int command (char *filename, register FILE *f)
 		xprintf ("\n");
 		if (clreol)
 			cleareol ();
-		xprintf ("...back %d page", nlines);
+		xprintf (_("...back %d page"), nlines);
 		if (nlines > 1)
 			pr ("s\n");
 		else
@@ -1076,7 +1217,7 @@ int command (char *filename, register FILE *f)
 	    ret (nscroll);
 	case 'q':
 	case 'Q':
-	    end_it ();
+	    end_it (0);
 	case 's':
 	case 'f':
 	    if (nlines == 0) nlines++;
@@ -1087,7 +1228,7 @@ int command (char *filename, register FILE *f)
 	    xprintf ("\n");
 	    if (clreol)
 		cleareol ();
-	    xprintf ("...skipping %d line", nlines);
+	    xprintf (_("...skipping %d line"), nlines);
 	    if (nlines > 1)
 		pr ("s\n");
 	    else
@@ -1122,19 +1263,19 @@ int command (char *filename, register FILE *f)
 		ret (dlines);
 	    }
 	    else {
-		write (2, &my_bell, 1);
+		ringbell();
 		break;
 	    }
 	case '\'':
 	    if (!no_intty) {
 		kill_line ();
-		pr ("\n***Back***\n\n");
+		pr (_("\n***Back***\n\n"));
 		Fseek (f, context.chrctr);
 		Currline = context.line;
 		ret (dlines);
 	    }
 	    else {
-		write (2, &my_bell, 1);
+		ringbell();
 		break;
 	    }
 	case '=':
@@ -1151,12 +1292,12 @@ int command (char *filename, register FILE *f)
 	    promptlen = 1;
 	    fflush (stdout);
 	    if (lastp) {
-		write (2,"\r", 1);
+		errwrite ("\r");
 		search (NULL, f, nlines);	/* Use previous r.e. */
 	    }
 	    else {
 		ttyin (cmdbuf, sizeof(cmdbuf)-2, '/');
-		write (2, "\r", 1);
+		errwrite("\r");
 		search (cmdbuf, f, nlines);
 	    }
 	    ret (dlines-1);
@@ -1165,8 +1306,27 @@ int command (char *filename, register FILE *f)
 	    break;
 	case '?':
 	case 'h':
-	    if ((helpf = fopen (HELPFILE, "r")) == NULL)
-		error ("Can't open help file");
+	    helpf = NULL;
+	    {
+		    char *lang;
+		    char hlpfile[sizeof(MOREHELPFILE)+4];
+
+		    lang = getenv("LANGUAGE");
+		    if (!lang || *lang == '\0')
+			    lang = setlocale(LC_MESSAGES, "");
+		    if (!lang || *lang == '\0')
+			    lang = getenv("LANG");
+		    if (lang && strlen(lang) > 1) {
+			    strcpy(hlpfile, MOREHELPFILE);
+			    strcat(hlpfile, ".");
+			    strncat(hlpfile, lang, 2);
+			    helpf = fopen (hlpfile, "r");
+		    }
+	    }
+	    if (helpf == NULL)
+		    helpf = fopen (MOREHELPFILE, "r");
+	    if (helpf == NULL)
+		error (_("Can't open help file"));
 	    if (noscroll) doclear ();
 	    copy_file (helpf);
 	    fclose (helpf);
@@ -1174,28 +1334,37 @@ int command (char *filename, register FILE *f)
 	    break;
 	case 'v':	/* This case should go right before default */
 	    if (!no_intty) {
-		kill_line ();
-		cmdbuf[0] = '+';
-		scanstr (Currline - dlines < 0 ? 0
-				: Currline - (dlines + 1) / 2, &cmdbuf[1]);
-		pr ("vi "); pr (cmdbuf); putchar (' '); pr (fnames[fnum]);
-		execute (filename, VI, "vi", cmdbuf, fnames[fnum], 0);
-		break;
+		    char *editor;
+
+		    editor = getenv("VISUAL");
+		    if (editor == NULL || *editor == '\0')
+			    editor = getenv("EDITOR");
+		    if (editor == NULL || *editor == '\0')
+			    editor = VI;
+
+		    kill_line ();
+		    cmdbuf[0] = '+';
+		    scanstr (Currline - dlines <= 0 ? 1
+			     : Currline - (dlines + 1) / 2, &cmdbuf[1]);
+		    pr (editor); putchar (' ');
+		    pr (cmdbuf); putchar (' '); pr (fnames[fnum]);
+		    execute (filename, editor, editor, cmdbuf, fnames[fnum], 0);
+		    break;
 	    }
 	default:
 	    if (dum_opt) {
    		kill_line ();
 		if (Senter && Sexit) {
-		    putp (Senter);
-		    promptlen = pr ("[Press 'h' for instructions.]") + (2 * soglitch);
-		    putp (Sexit);
+		    my_putstring (Senter);
+		    promptlen = pr (_("[Press 'h' for instructions.]")) + (2 * soglitch);
+		    my_putstring (Sexit);
 		}
 		else
-		    promptlen = pr ("[Press 'h' for instructions.]");
+		    promptlen = pr (_("[Press 'h' for instructions.]"));
 		fflush (stdout);
 	    }
 	    else
-		write (2, &my_bell, 1);
+		ringbell();
 	    break;
 	}
 	if (done) break;
@@ -1207,7 +1376,7 @@ endsw:
     return (retval);
 }
 
-char ch;
+static char ch;
 
 /*
  * Execute a colon-prefixed command.
@@ -1215,8 +1384,7 @@ char ch;
  * more of the file to be printed.
  */
 
-int colon (char *filename, int cmd, int nlines)
-{
+int colon (char *filename, int cmd, int nlines) {
 	if (cmd == 0)
 		ch = readch ();
 	else
@@ -1226,15 +1394,15 @@ int colon (char *filename, int cmd, int nlines)
 	case 'f':
 		kill_line ();
 		if (!no_intty)
-			promptlen = xprintf ("\"%s\" line %d", fnames[fnum], Currline);
+			promptlen = xprintf (_("\"%s\" line %d"), fnames[fnum], Currline);
 		else
-			promptlen = xprintf ("[Not a file] line %d", Currline);
+			promptlen = xprintf (_("[Not a file] line %d"), Currline);
 		fflush (stdout);
 		return (-1);
 	case 'n':
 		if (nlines == 0) {
 			if (fnum >= nfiles - 1)
-				end_it ();
+				end_it (0);
 			nlines++;
 		}
 		putchar ('\r');
@@ -1243,7 +1411,7 @@ int colon (char *filename, int cmd, int nlines)
 		return (0);
 	case 'p':
 		if (no_intty) {
-			write (2, &my_bell, 1);
+			ringbell();
 			return (-1);
 		}
 		putchar ('\r');
@@ -1257,9 +1425,9 @@ int colon (char *filename, int cmd, int nlines)
 		return (-1);
 	case 'q':
 	case 'Q':
-		end_it ();
+		end_it (0);
 	default:
-		write (2, &my_bell, 1);
+		ringbell();
 		return (-1);
 	}
 }
@@ -1276,9 +1444,9 @@ int number(char *cmd)
 	i = 0; ch = otty.c_cc[VKILL];
 	for (;;) {
 		ch = readch ();
-		if (ch >= '0' && ch <= '9')
+		if (isdigit(ch))
 			i = i*10 + ch - '0';
-		else if (ch == otty.c_cc[VKILL])
+		else if ((cc_t) ch == otty.c_cc[VKILL])
 			i = 0;
 		else {
 			*cmd = ch;
@@ -1290,7 +1458,9 @@ int number(char *cmd)
 
 void do_shell (char *filename)
 {
-	char cmdbuf[80];
+	char cmdbuf[200];
+	int rc;
+	char *expanded;
 
 	kill_line ();
 	pr ("!");
@@ -1300,13 +1470,26 @@ void do_shell (char *filename)
 		pr (shell_line);
 	else {
 		ttyin (cmdbuf, sizeof(cmdbuf)-2, '!');
-		if (expand (shell_line, cmdbuf)) {
+		expanded = 0;
+		rc = expand (&expanded, cmdbuf);
+		if (expanded) {
+			if (strlen(expanded) < sizeof(shell_line))
+				strcpy(shell_line, expanded);
+			else
+				rc = -1;
+			free(expanded);
+		}
+		if (rc < 0) {
+			errwrite(_("  Overflow\n"));
+			prompt (filename);
+			return;
+		} else if (rc > 0) {
 			kill_line ();
 			promptlen = xprintf ("!%s", shell_line);
 		}
 	}
 	fflush (stdout);
-	write (2, "\n", 1);
+	errwrite("\n");
 	promptlen = 0;
 	shellp = 1;
 	execute (filename, shell, shell, "-c", shell_line, 0);
@@ -1323,8 +1506,8 @@ void search(char buf[], FILE *file, register int n)
     register long line2 = startline;
     register long line3 = startline;
     register int lncount;
-    int saveln, rv, re_exec();
-    char *s, *re_comp();
+    int saveln, rv;
+    char *s;
 
     context.line = saveln = Currline;
     context.chrctr = startline;
@@ -1337,42 +1520,44 @@ void search(char buf[], FILE *file, register int n)
 	line1 = Ftell (file);
 	rdline (file);
 	lncount++;
-	if ((rv = re_exec (Line)) == 1)
+	if ((rv = re_exec (Line)) == 1) {
 		if (--n == 0) {
 		    if (lncount > 3 || (lncount > 1 && no_intty))
 		    {
 			pr ("\n");
 			if (clreol)
 			    cleareol ();
-			pr("...skipping\n");
+			pr(_("...skipping\n"));
 		    }
 		    if (!no_intty) {
 			Currline -= (lncount >= 3 ? 3 : lncount);
 			Fseek (file, line3);
-			if (noscroll)
+			if (noscroll) {
 			    if (clreol) {
 				home ();
 				cleareol ();
 			    }
 			    else
 				doclear ();
+			}
 		    }
 		    else {
 			kill_line ();
-			if (noscroll)
+			if (noscroll) {
 			    if (clreol) {
 			        home ();
 			        cleareol ();
 			    }
 			    else
 				doclear ();
+			}
 			pr (Line);
 			putchar ('\n');
 		    }
 		    break;
 		}
-	else if (rv == -1)
-	    error ("Regular expression botch");
+	} else if (rv == -1)
+	    error (_("Regular expression botch"));
     }
     if (feof (file)) {
 	if (!no_intty) {
@@ -1384,10 +1569,10 @@ void search(char buf[], FILE *file, register int n)
 	    Fseek (file, startline);
 	}
 	else {
-	    pr ("\nPattern not found\n");
-	    end_it ();
+	    pr (_("\nPattern not found\n"));
+	    end_it (0);
 	}
-	error ("Pattern not found");
+	error (_("Pattern not found"));
     }
 }
 
@@ -1412,14 +1597,13 @@ void execute (char * filename, char * cmd, ...)
 	    }
 
 	    va_start(argp, cmd);
-
 	    arg = va_arg(argp, char *);
 	    argcount = 0;
 	    while (arg) {
 		argcount++;
 	        arg = va_arg(argp, char *);
 	    }
-	    va_end(argp);	/* balance {}'s for some UNIX's */
+	    va_end(argp);
 
 	    args = alloca(sizeof(char *) * (argcount + 1));
 	    args[argcount] = NULL;
@@ -1432,10 +1616,10 @@ void execute (char * filename, char * cmd, ...)
 		argcount++;
 	        arg = va_arg(argp, char *);
 	    }
-	    va_end(argp);	/* balance {}'s for some UNIX's */
+	    va_end(argp);
 	
-	    execv (cmd, args);
-	    write (2, "exec failed\n", 12);
+	    execvp (cmd, args);
+	    errwrite(_("exec failed\n"));
 	    exit (1);
 	}
 	if (id > 0) {
@@ -1449,7 +1633,7 @@ void execute (char * filename, char * cmd, ...)
 	    if (catch_susp)
 		signal(SIGTSTP, onsusp);
 	} else
-	    write(2, "can't fork\n", 11);
+	    errwrite(_("can't fork\n"));
 	set_tty ();
 	pr ("------------------------\n");
 	prompt (filename);
@@ -1460,7 +1644,7 @@ void execute (char * filename, char * cmd, ...)
 
 void skiplns (register int n, register FILE *f)
 {
-    register char c;
+    register int c;
 
     while (n > 0) {
 	while ((c = Getc (f)) != '\n')
@@ -1488,12 +1672,12 @@ void skipf (register int nskip)
     fnum += nskip;
     if (fnum < 0)
 	fnum = 0;
-    pr ("\n...Skipping ");
+    pr (_("\n...Skipping "));
     pr ("\n");
     if (clreol)
 	cleareol ();
-    pr ("...Skipping ");
-    pr (nskip > 0 ? "to file " : "back to file ");
+    pr (_("...Skipping "));
+    pr (nskip > 0 ? _("to file ") : _("back to file "));
     pr (fnames[fnum]);
     pr ("\n");
     if (clreol)
@@ -1538,7 +1722,7 @@ retry:
 	if ((term = getenv("TERM")) == 0) {
 	    dumb++; ul_opt = 0;
 	}
-        setupterm(term, 1, &ret);
+        my_setupterm(term, 1, &ret);
 	if (ret <= 0) {
 	    dumb++; ul_opt = 0;
 	}
@@ -1546,35 +1730,35 @@ retry:
 #ifdef TIOCGWINSZ
 	    if (ioctl(fileno(stdout), TIOCGWINSZ, &win) < 0) {
 #endif
-		Lpp = tigetnum("lines");
-		Mcol = tigetnum("cols");
+		Lpp = my_tgetnum("li","lines");
+		Mcol = my_tgetnum("co","cols");
 #ifdef TIOCGWINSZ
 	    } else {
 		if ((Lpp = win.ws_row) == 0)
-		    Lpp = tigetnum("lines");
+		    Lpp = my_tgetnum("li","lines");
 		if ((Mcol = win.ws_col) == 0)
-		    Mcol = tigetnum("cols");
+		    Mcol = my_tgetnum("co","cols");
 	    }
 #endif
-	    if ((Lpp <= 0) || tigetflag("hc")) {
+	    if ((Lpp <= 0) || my_tgetflag("hc","hc")) {
 		hard++;	/* Hard copy terminal */
 		Lpp = 24;
 	    }
 
-	    if (tigetflag("xenl"))
+	    if (my_tgetflag("xn","xenl"))
 		eatnl++; /* Eat newline at last column + 1; dec, concept */
 	    if (Mcol <= 0)
 		Mcol = 80;
 
 	    if (tailequ (fnames[0], "page"))
 		noscroll++;
-	    Wrap = tigetflag("am");
-	    bad_so = tigetflag ("xhp");
-	    eraseln = tigetstr("el");
-	    Clear = tigetstr("clear");
-	    Senter = tigetstr("smso");
-	    Sexit = tigetstr("rmso");
-	    if ((soglitch = tigetnum("xmc")) < 0)
+	    Wrap = my_tgetflag("am","am");
+	    bad_so = my_tgetflag ("xs","xhp");
+	    eraseln = my_tgetstr("ce","el");
+	    Clear = my_tgetstr("cl","clear");
+	    Senter = my_tgetstr("so","smso");
+	    Sexit = my_tgetstr("se","rmso");
+	    if ((soglitch = my_tgetnum("sg","xmc")) < 0)
 		soglitch = 0;
 
 	    /*
@@ -1585,12 +1769,12 @@ retry:
 	     *  isn't available, settle for standout sequence.
 	     */
 
-	    if (tigetflag("ul") || tigetflag("os"))
+	    if (my_tgetflag("ul","ul") || my_tgetflag("os","os"))
 		ul_opt = 0;
-	    if ((chUL = tigetstr("uc")) == NULL )
+	    if ((chUL = my_tgetstr("uc","uc")) == NULL )
 		chUL = "";
-	    if (((ULenter = tigetstr("smul")) == NULL ||
-	         (ULexit = tigetstr("rmul")) == NULL) && !*chUL) {
+	    if (((ULenter = my_tgetstr("us","smul")) == NULL ||
+	         (ULexit = my_tgetstr("ue","rmul")) == NULL) && !*chUL) {
 	        if ((ULenter = Senter) == NULL || (ULexit = Sexit) == NULL) {
 			ULenter = "";
 			ULexit = "";
@@ -1600,18 +1784,18 @@ retry:
 		ulglitch = 0;
 	    }
 
-	    if ((padstr = tigetstr("pad")) != NULL)
+	    if ((padstr = my_tgetstr("pc","pad")) != NULL)
 		PC = *padstr;
-	    Home = tigetstr("home");
-	    if (Home == 0 || *Home == '\0')
-	    {
-		if ((cursorm = tigetstr("cup")) != NULL) {
-		    strcpy(cursorhome, (const char *)tparm(cursorm, 0, 0));
+	    Home = my_tgetstr("ho","home");
+	    if (Home == 0 || *Home == '\0') {
+		if ((cursorm = my_tgetstr("cm","cup")) != NULL) {
+		    const char *t = (const char *)my_tgoto(cursorm, 0, 0);
+		    xstrncpy(cursorhome, t, sizeof(cursorhome));
 		    Home = cursorhome;
-	       }
+		}
 	    }
-	    EodClr = tigetstr("ed");
-	    if ((chBS = tigetstr("cub1")) == NULL)
+	    EodClr = my_tgetstr("cd","ed");
+	    if ((chBS = my_tgetstr("le","cub1")) == NULL)
 		chBS = "\b";
 
 	}
@@ -1630,51 +1814,49 @@ retry:
     }
 }
 
-int readch ()
-{
-	char ch;
-	extern int errno;
+int readch () {
+	char c;
 
 	errno = 0;
-	if (read (2, &ch, 1) <= 0)
+	if (read (2, &c, 1) <= 0) {
 		if (errno != EINTR)
-			end_it();
+			end_it(0);
 		else
-			ch = otty.c_cc[VKILL];
-	return (ch);
+			c = otty.c_cc[VKILL];
+	}
+	return (c);
 }
 
-static char BS = '\b';
+static char *BS = "\b";
 static char *BSB = "\b \b";
-static char CARAT = '^';
+static char *CARAT = "^";
 #define ERASEONECHAR \
     if (docrterase) \
-	write (2, BSB, sizeof(BSB)); \
+	errwrite(BSB); \
     else \
-	write (2, &BS, sizeof(BS));
+	errwrite(BS);
 
-void ttyin (char buf[], register int nmax, char pchar)
-{
-    register char *sptr;
-    register char ch;
-    register int slash = 0;
+void ttyin (char buf[], register int nmax, char pchar) {
+    char *sp;
+    char c;
+    int slash = 0;
     int	maxlen;
     char cbuf;
 
-    sptr = buf;
+    sp = buf;
     maxlen = 0;
-    while (sptr - buf < nmax) {
+    while (sp - buf < nmax) {
 	if (promptlen > maxlen) maxlen = promptlen;
-	ch = readch ();
-	if (ch == '\\') {
+	c = readch ();
+	if (c == '\\') {
 	    slash++;
 	}
-	else if ((ch == otty.c_cc[VERASE]) && !slash) {
-	    if (sptr > buf) {
+	else if (((cc_t) c == otty.c_cc[VERASE]) && !slash) {
+	    if (sp > buf) {
 		--promptlen;
 		ERASEONECHAR
-		--sptr;
-		if ((*sptr < ' ' && *sptr != '\n') || *sptr == RUBOUT) {
+		--sp;
+		if ((*sp < ' ' && *sp != '\n') || *sp == RUBOUT) {
 		    --promptlen;
 		    ERASEONECHAR
 		}
@@ -1682,12 +1864,12 @@ void ttyin (char buf[], register int nmax, char pchar)
 	    }
 	    else {
 		if (!eraseln) promptlen = maxlen;
-		longjmp (restore, 1);
+		siglongjmp (restore, 1);
 	    }
 	}
-	else if ((ch == otty.c_cc[VKILL]) && !slash) {
+	else if (((cc_t) c == otty.c_cc[VKILL]) && !slash) {
 	    if (hard) {
-		show (ch);
+		show (c);
 		putchar ('\n');
 		putchar (pchar);
 	    }
@@ -1698,94 +1880,122 @@ void ttyin (char buf[], register int nmax, char pchar)
 		    erasep (1);
 		else if (docrtkill)
 		    while (promptlen-- > 1)
-			write (2, BSB, sizeof(BSB));
+			errwrite(BSB);
 		promptlen = 1;
 	    }
-	    sptr = buf;
+	    sp = buf;
 	    fflush (stdout);
 	    continue;
 	}
-	if (slash && (ch == otty.c_cc[VKILL] || ch == otty.c_cc[VERASE])) {
+	if (slash && ((cc_t) c == otty.c_cc[VKILL]
+		   || (cc_t) c == otty.c_cc[VERASE])) {
 	    ERASEONECHAR
-	    --sptr;
+	    --sp;
 	}
-	if (ch != '\\')
+	if (c != '\\')
 	    slash = 0;
-	*sptr++ = ch;
-	if ((ch < ' ' && ch != '\n' && ch != ESC) || ch == RUBOUT) {
-	    ch += ch == RUBOUT ? -0100 : 0100;
-	    write (2, &CARAT, 1);
+	*sp++ = c;
+	if ((c < ' ' && c != '\n' && c != ESC) || c == RUBOUT) {
+	    c += (c == RUBOUT) ? -0100 : 0100;
+	    errwrite(CARAT);
 	    promptlen++;
 	}
-	cbuf = ch;
-	if (ch != '\n' && ch != ESC) {
-	    write (2, &cbuf, 1);
+	cbuf = c;
+	if (c != '\n' && c != ESC) {
+	    errwrite1(&cbuf);
 	    promptlen++;
 	}
 	else
 	    break;
     }
-    *--sptr = '\0';
+    *--sp = '\0';
     if (!eraseln) promptlen = maxlen;
-    if (sptr - buf >= nmax - 1)
-	error ("Line too long");
+    if (sp - buf >= nmax - 1)
+	error (_("Line too long"));
 }
 
-int expand (char *outbuf, char *inbuf)
-{
-    register char *instr;
-    register char *outstr;
-    register char ch;
-    char temp[200];
+/* return: 0 - unchanged, 1 - changed, -1 - overflow (unchanged) */
+int expand (char **outbuf, char *inbuf) {
+    char *inpstr;
+    char *outstr;
+    char c;
+    char *temp;
     int changed = 0;
+    int tempsz, xtra, offset;
 
-    instr = inbuf;
+    xtra = strlen (fnames[fnum]) + strlen (shell_line) + 1;
+    tempsz = 200 + xtra;
+    temp = malloc(tempsz);
+    if (!temp) {
+	    error (_("Out of memory"));
+	    return -1;
+    }
+    inpstr = inbuf;
     outstr = temp;
-    while ((ch = *instr++) != '\0')
-	switch (ch) {
+    while ((c = *inpstr++) != '\0'){
+	offset = outstr-temp;
+	if (tempsz-offset-1 < xtra) {
+		tempsz += 200 + xtra;
+		temp = realloc(temp, tempsz);
+		if (!temp) {
+			error (_("Out of memory"));
+			return -1;
+		}
+		outstr = temp + offset;
+	}
+	switch (c) {
 	case '%':
 	    if (!no_intty) {
 		strcpy (outstr, fnames[fnum]);
 		outstr += strlen (fnames[fnum]);
 		changed++;
-	    }
-	    else
-		*outstr++ = ch;
+	    } else
+		*outstr++ = c;
 	    break;
 	case '!':
 	    if (!shellp)
-		error ("No previous command to substitute for");
+		error (_("No previous command to substitute for"));
 	    strcpy (outstr, shell_line);
 	    outstr += strlen (shell_line);
 	    changed++;
 	    break;
 	case '\\':
-	    if (*instr == '%' || *instr == '!') {
-		*outstr++ = *instr++;
+	    if (*inpstr == '%' || *inpstr == '!') {
+		*outstr++ = *inpstr++;
 		break;
 	    }
 	default:
-	    *outstr++ = ch;
+	    *outstr++ = c;
 	}
+    }
     *outstr++ = '\0';
-    strcpy (outbuf, temp);
+    *outbuf = temp;
     return (changed);
 }
 
-void show (register char ch)
-{
+void show (char c) {
     char cbuf;
 
-    if ((ch < ' ' && ch != '\n' && ch != ESC) || ch == RUBOUT) {
-	ch += ch == RUBOUT ? -0100 : 0100;
-	write (2, &CARAT, 1);
+    if ((c < ' ' && c != '\n' && c != ESC) || c == RUBOUT) {
+	c += (c == RUBOUT) ? -0100 : 0100;
+	errwrite(CARAT);
 	promptlen++;
     }
-    cbuf = ch;
-    write (2, &cbuf, 1);
+    cbuf = c;
+    errwrite1(&cbuf);
     promptlen++;
 }
 
+void errwrite (char *txt)
+{
+    write (fileno(stderr), txt, strlen(txt));
+}
+
+void errwrite1 (char *sym)
+{
+    write (fileno(stderr), sym, 1);
+}
+
 void error (char *mess)
 {
     if (clreol)
@@ -1794,30 +2004,32 @@ void error (char *mess)
 	kill_line ();
     promptlen += strlen (mess);
     if (Senter && Sexit) {
-	putp (Senter);
+	my_putstring (Senter);
 	pr(mess);
-	putp (Sexit);
+	my_putstring (Sexit);
     }
     else
 	pr (mess);
     fflush(stdout);
     errors++;
-    longjmp (restore, 1);
+    siglongjmp (restore, 1);
 }
 
 
-void set_tty ()
-{
+void set_tty () {
 	otty.c_lflag &= ~(ICANON|ECHO);
+	otty.c_cc[VMIN] = 1;	/* read at least 1 char */
+	otty.c_cc[VTIME] = 0;	/* no timeout */
 	stty(fileno(stderr), &otty);
 }
 
-int ourputch(int ch) {
-    return putc(ch, stdout);
+static int
+ourputch(int c) {
+    return putc(c, stdout);
 }
 
-void reset_tty ()
-{
+void
+reset_tty () {
     if (no_tty)
 	return;
     if (pstate) {
@@ -1826,12 +2038,14 @@ void reset_tty ()
 	pstate = 0;
     }
     otty.c_lflag |= ICANON|ECHO;
+    otty.c_cc[VMIN] = savetty0.c_cc[VMIN];
+    otty.c_cc[VTIME] = savetty0.c_cc[VTIME];
     stty(fileno(stderr), &savetty0);
 }
 
 void rdline (register FILE *f)
 {
-    register char c;
+    register int  c;
     register char *p;
 
     p = Line;
@@ -1844,8 +2058,7 @@ void rdline (register FILE *f)
 
 /* Come here when we get a suspend signal from the terminal */
 
-void onsusp ()
-{
+void onsusp (int dummy) {
     sigset_t signals, oldmask;
 
     /* ignore SIGTTOU so we don't get stopped if csh grabs the tty */
@@ -1870,5 +2083,5 @@ void onsusp ()
     signal (SIGTSTP, onsusp);
     set_tty ();
     if (inwait)
-	    longjmp (restore, 1);
+	    siglongjmp (restore, 1);
 }
