@@ -31,9 +31,9 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static char sccsid[] = "@(#)odsyntax.c	5.4 (Berkeley) 3/8/91";
-#endif /* not lint */
+ /* 1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+  * - added Native Language Support
+  */
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -41,8 +41,10 @@ static char sccsid[] = "@(#)odsyntax.c	5.4 (Berkeley) 3/8/91";
 #include <ctype.h>		/* for isdigit() */
 #include <unistd.h>		/* for getopt() */
 #include "hexdump.h"
+#include "nls.h"
 
 static void odoffset(int, char ***);
+static void odprecede(void);
 
 int deprecated;
 
@@ -54,7 +56,6 @@ oldsyntax(int argc, char ***argvp)
 	extern int optind;
 	int ch;
 	char **argv;
-	static void odprecede();
 
 	deprecated = 1;
 	argv = *argvp;
@@ -129,11 +130,11 @@ oldsyntax(int argc, char ***argvp)
 		case '?':
 		default:
 			(void)fprintf(stderr,
-			    "od: od(1) has been deprecated for hexdump(1).\n");
+			    _("od: od(1) has been deprecated for hexdump(1).\n"));
 			if (ch != '?')
 				(void)fprintf(stderr,
-"od: hexdump(1) compatibility doesn't support the -%c option%s\n",
-				    ch, ch == 's' ? "; see strings(1)." : ".");
+_("od: hexdump(1) compatibility doesn't support the -%c option%s\n"),
+				    ch, ch == 's' ? _("; see strings(1).") : ".");
 			usage();
 		}
 
@@ -243,7 +244,7 @@ odoffset(int argc, char ***argvp)
 }
 
 static void
-odprecede()
+odprecede(void)
 {
 	static int first = 1;
 
